@@ -37,7 +37,7 @@ static audio_change change;
 static int init(struct xmp_context *);
 static int setaudio(struct xmp_options *);
 static void bufdump(struct xmp_context *, int);
-static void shutdown(void);
+static void shutdown(struct xmp_context *);
 
 static void dummy()
 {
@@ -163,9 +163,9 @@ static void bufdump(struct xmp_context *ctx, int i)
 	};
 }
 
-static void shutdown()
+static void shutdown(struct xmp_context *ctx)
 {
-	xmp_smix_off();
+	xmp_smix_off(ctx);
 	control.ioctl_request = AUDIO_STOP;
 	ioctl(audio_fd, AUDIO_CONTROL, &control);
 	close(audio_fd);
