@@ -22,15 +22,14 @@
 
 static snd_seq_t *seq;
 static int queue;
-//static int echo_msg;
 static int my_client, my_port;
 static int dest_client, dest_port;
 static int nvoices;
 static snd_hwdep_t *hwdep;
 
-static int numvoices	(int);
+static int numvoices	(struct xmp_context *, int);
 static void voicepos	(int, int);
-static void echoback	(int);
+static void echoback	(struct xmp_context *, int);
 static void setpatch	(int, int);
 static void setvol	(int, int);
 static void setnote	(int, int);
@@ -47,7 +46,7 @@ static void seq_sync	(double);
 static int writepatch	(struct patch_info *);
 static int init(struct xmp_context *ctx);
 static int getmsg	(void);
-static void shutdown	(void);
+static void shutdown	(struct xmp_context *);
 
 static char *help[] = {
 	"addr=port", "Set ALSA sequencer port",
@@ -172,7 +171,7 @@ static void midi_send(snd_seq_event_t *ev)
 }
 
 
-static int numvoices(int num)
+static int numvoices(struct xmp_context *ctx, int num)
 {
 	return nvoices;
 }
@@ -184,7 +183,7 @@ static void voicepos(int ch, int pos)
 }
 
 
-static void echoback(int msg)
+static void echoback(struct xmp_context *ctx, int msg)
 {
 	//SEQ_ECHO_BACK (msg);
 }
@@ -455,7 +454,7 @@ error:
 }
 
 
-static void shutdown ()
+static void shutdown(struct xmp_context *ctx)
 {
 	snd_seq_free_queue(seq, queue);
 	snd_hwdep_close(hwdep);
