@@ -26,7 +26,7 @@ static int fd;
 
 static int init(struct xmp_context *);
 static void bufdump(struct xmp_context *, int);
-static void shutdown();
+static void shutdown(struct xmp_context *);
 
 static void dummy()
 {
@@ -101,9 +101,9 @@ static void bufdump(struct xmp_context *ctx, int i)
 	}
 }
 
-static void shutdown()
+static void shutdown(struct xmp_context *ctx)
 {
-	xmp_smix_off();
+	xmp_smix_off(ctx);
 
 	if (fd)
 		close(fd);
