@@ -19,7 +19,7 @@ static arts_stream_t as;
 
 static int init(struct xmp_context *);
 static void bufdump(struct xmp_context *, int);
-static void myshutdown();
+static void myshutdown(struct xmp_context *);
 
 static void dummy()
 {
@@ -89,9 +89,9 @@ static void bufdump(struct xmp_context *ctx, int i)
 	} while (i);
 }
 
-static void myshutdown()
+static void myshutdown(struct xmp_context *ctx)
 {
-	xmp_smix_off();
+	xmp_smix_off(ctx);
 	arts_close_stream(as);
 	arts_free();
 }
