@@ -30,7 +30,7 @@ static int endian;
 
 static int init(struct xmp_context *);
 static void bufdump(struct xmp_context *, int);
-static void shutdown();
+static void shutdown(struct xmp_context *);
 
 static void dummy()
 {
@@ -58,7 +58,7 @@ struct xmp_drv_info drv_file = {
 	dummy,			/* setbend */
 	xmp_smix_seteffect,	/* seteffect */
 	dummy,			/* starttimer */
-	dummy,			/* flush */
+	dummy,			/* stoptimer */
 	dummy,			/* resetvoices */
 	bufdump,		/* bufdump */
 	dummy,			/* bufwipe */
@@ -78,8 +78,8 @@ static int init(struct xmp_context *ctx)
 
 	endian = 0;
 	parm_init();
-	chkparm1("big-endian", endian = 1);
-	chkparm1("little-endian", endian = -1);
+	chkparm0("big-endian", endian = 1);
+	chkparm0("little-endian", endian = -1);
 	parm_end();
 
 	if (!o->outfile)
@@ -126,9 +126,9 @@ static void bufdump(struct xmp_context *ctx, int i)
 	}
 }
 
-static void shutdown()
+static void shutdown(struct xmp_context *ctx)
 {
-	xmp_smix_off();
+	xmp_smix_off(ctx);
 
 	if (fd)
 		close(fd);
