@@ -14,7 +14,7 @@
 #include <CoreAudio/CoreAudio.h>
 #include <AudioUnit/AudioUnit.h>
 #include <AudioToolbox/AudioToolbox.h>
-#include <unistd.h>
+#include <CoreServices/CoreServices.h>
 
 #include "common.h"
 #include "driver.h"
@@ -23,7 +23,7 @@
 
 static int init (struct xmp_context *ctx);
 static void bufdump (struct xmp_context *, int);
-static void shutdown (void);
+static void shutdown (struct xmp_context *);
 
 static void dummy () { }
 
@@ -289,9 +289,9 @@ static void bufdump(struct xmp_context *ctx, int i)
 }
 
 
-static void shutdown ()
+static void shutdown(struct xmp_context *ctx)
 {
-	xmp_smix_off();
+	xmp_smix_off(ctx);
         AudioOutputUnitStop(au);
 	AudioUnitUninitialize(au);
 	CloseComponent(au);
