@@ -54,7 +54,7 @@ static int audioctl_fd;
 static int init(struct xmp_context *);
 static int setaudio(struct xmp_options *);
 static void bufdump(struct xmp_context *, int);
-static void shutdown(void);
+static void shutdown(struct xmp_context *);
 
 static void dummy()
 {
@@ -219,8 +219,8 @@ static void bufdump(struct xmp_context *ctx, int i)
 	}
 }
 
-static void shutdown()
+static void shutdown(struct xmp_context *ctx)
 {
-	xmp_smix_off();
+	xmp_smix_off(ctx);
 	close(audio_fd);
 }
