@@ -234,7 +234,7 @@ static int ssn_load(struct xmp_context *ctx, FILE *f, const int start)
     for (i = 0; i < m->xxh->chn; i++)
 	m->xxc[i].pan = (i % 2) * 0xff;
 
-    m->fetch |= XMP_CTL_PERPAT;	    /* Cancel persistent fx at each new pat */
+    m->quirk |= XMP_QRK_PERPAT;	    /* Cancel persistent fx at each new pat */
 
     return 0;
 }
