@@ -27,7 +27,7 @@ static int amf_load (struct xmp_context *, FILE *, const int);
 
 struct xmp_loader_info amf_loader = {
 	"AMF",
-	"DSMI (DMP)",
+	"DSMI Advanced Module Format",
 	amf_test,
 	amf_load
 };
@@ -68,7 +68,7 @@ static int amf_load(struct xmp_context *ctx, FILE *f, const int start)
 
 	fread(buf, 1, 32, f);
 	strncpy(m->name, (char *)buf, 32);
-	sprintf(m->type, "DSMI %d.%d (DMP)", ver / 10, ver % 10);
+	sprintf(m->type, "DSMI %d.%d", ver / 10, ver % 10);
 
 	m->xxh->ins = read8(f);
 	m->xxh->len = read8(f);
@@ -374,7 +374,7 @@ static int amf_load(struct xmp_context *ctx, FILE *f, const int start)
 
 	/* Samples */
 
-	if (V(0)) report ("Stored samples : %d ", m->xxh->smp);
+	reportv(ctx, 0, "Stored samples : %d ", m->xxh->smp);
 
 	for (i = 0; i < m->xxh->ins; i++) {
 		xmp_drv_loadpatch(ctx, f, m->xxi[i][0].sid, m->c4rate,
@@ -383,7 +383,7 @@ static int amf_load(struct xmp_context *ctx, FILE *f, const int start)
 	}
 	reportv(ctx, 0, "\n");
 
-	m->fetch |= XMP_CTL_FINEFX;
+	m->quirk |= XMP_QRK_FINEFX;
 
 	return 0;
 }
