@@ -1,7 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
- *
- * $Id: common.c,v 1.21 2007/12/05 12:24:42 cmatsuoka Exp $
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -15,7 +13,7 @@
 #include <ctype.h>
 
 #include "xmp.h"
-#include "xmpi.h"
+#include "common.h"
 #include "period.h"
 #include "load.h"
 
@@ -30,9 +28,10 @@ char *copy_adjust(uint8 *s, uint8 *r, int n)
 	memset(s, 0, n);
 	strncpy((char *)s, (char *)r, n);
 
-	for (i = 0; i < n; i++)
-		if (!isprint(s[i]) || ((uint8) s[i] > 127))
+	for (i = 0; i < n; i++) {
+		if (!isprint(s[i]) || ((uint8)s[i] > 127))
 			s[i] = '.';
+	}
 
 	while (*s && (s[strlen((char *)s) - 1] == ' '))
 		s[strlen((char *)s) - 1] = 0;
@@ -78,8 +77,6 @@ void set_xxh_defaults(struct xxm_header *xxh)
 	xxh->tpo = 6;
 	xxh->bpm = 125;
 	xxh->chn = 4;
-	xxh->ins = 31;
-	xxh->smp = xxh->ins;
 }
 
 void cvt_pt_event(struct xxm_event *event, uint8 *mod_event)
