@@ -29,11 +29,26 @@ void iff_chunk(struct xmp_context *ctx, FILE *f)
     long size;
     char id[17] = "";
 
-    if (fread (id, 1, __id_size, f) != __id_size)
+    if (fread(id, 1, __id_size, f) != __id_size)
 	return;
 
+    if (__flags & IFF_SKIP_EMBEDDED) {
+	/* embedded RIFF hack */
+	if (!strncmp(id, "RIFF", 4)) {
+	    read32b(f);
+	    read32b(f);
+	    fread(id, 1, __id_size, f);	/* read first chunk ID instead */
+        }
+    }
+
     size = (__flags & IFF_LITTLE_ENDIAN) ? read32l(f) : read32b(f);
 
+    if (__flags & IFF_CHUNK_ALIGN2)
+	size = (size + 1) & ~1;
+
+    if (__flags & IFF_CHUNK_ALIGN4)
+	size = (size + 3) & ~3;
+
     if (__flags & IFF_FULL_CHUNK_SIZE)
 	size -= __id_size + 4;
 
