@@ -1,7 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
- *
- * $Id: ptm_load.c,v 1.23 2007/11/29 17:43:20 cmatsuoka Exp $
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -176,7 +174,7 @@ static int ptm_load(struct xmp_context *ctx, FILE *f, const int start)
 	fseek(f, start + 16L * pfh.patseg[i], SEEK_SET);
 	r = 0;
 	while (r < 64) {
-	    fread (&b, 1, 1, f);
+	    b = read8(f);
 	    if (!b) {
 		r++;
 		continue;
@@ -188,7 +186,7 @@ static int ptm_load(struct xmp_context *ctx, FILE *f, const int start)
 
 	    event = &EVENT (i, c, r);
 	    if (b & PTM_NI_FOLLOW) {
-		fread (&n, 1, 1, f);
+		n = read8(f);
 		switch (n) {
 		case 255:
 		    n = 0;
@@ -198,8 +196,7 @@ static int ptm_load(struct xmp_context *ctx, FILE *f, const int start)
 		    break;	/* Key off */
 		}
 		event->note = n;
-		fread (&n, 1, 1, f);
-		event->ins = n;
+		event->ins = read8(f);
 	    }
 	    if (b & PTM_FX_FOLLOWS) {
 		event->fxt = read8(f);
