@@ -29,6 +29,9 @@
 #include <time.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#ifdef __CYGWIN__
+#include <sys/select.h>
+#endif
 #ifdef HAVE_SIGNAL_H
 #include <signal.h>
 #endif
@@ -72,6 +75,7 @@ int probeonly = 0;
 int randomize = 0;
 int loadonly = 0;
 int nocmd = 0;
+int showtime = 0;
 #ifdef HAVE_SYS_RTPRIO_H
 int rt = 0;
 #endif
@@ -96,8 +100,10 @@ static int sigusr = 0;
 #endif
 
 void get_options (int, char **, struct xmp_options *, xmp_context);
+void init_drivers (void);
 
 static xmp_context ctx;
+static int paused;
 
 
 static int set_tty()
@@ -129,8 +135,8 @@ static int reset_tty()
     if (background)
 	return -1;
 
-    if (tcsetattr (0, TCSAFLUSH, &term) < 0) {
-	fprintf (stderr, "can't reset terminal!\n");
+    if (tcsetattr(0, TCSAFLUSH, &term) < 0) {
+	fprintf(stderr, "can't reset terminal!\n");
 	return -1;
     }
 #endif
@@ -145,7 +151,10 @@ static int reset_tty()
 static void sigtstp_handler()
 {
     if (!stopped) {
-	fprintf (stderr, "] - STOPPED\n");
+	if (showtime)
+	    fprintf(stderr, "\n");
+	else if (verbosity)
+	    fprintf(stderr, "] - STOPPED\n");
 	xmp_timer_stop(ctx);
 	stopped = 1;
     }
@@ -207,29 +216,63 @@ static void cleanup(int s)
 
 #endif /* HAVE_SIGNAL_H */
 
+#ifdef __CYGWIN__
+/*
+ * from	daniel åkerud <daniel.akerud@gmail.com>
+ * date	Tue, Jul 28, 2009 at 9:59 AM
+ *
+ * Under Cygwin, the read() in process_echoback blocks because VTIME = 0
+ * is not handled correctly. To fix this you can either:
+ *
+ * 1. Enable "tty emulation" in Cygwin using an environment variable.
+ * http://www.mail-archive.com/cygwin@cygwin.com/msg99417.html
+ * For me this is _very_ slow and I can see the characters as they are
+ * typed out when running xmp. I have not investigated why this is
+ * happening, but there is of course a reason why this mode is not
+ * enabled by default.
+ * 
+ * 2. Do a select() before read()ing if the platform is Cygwin.
+ * This makes Cygwin builds work out of the box with no fiddling around,
+ * but does impose a neglectible cpu overhead (for Cygwin builds only).
+ */
+static int stdin_ready_for_reading()
+{
+    fd_set fds;
+    struct timeval tv;
+    int ret;
+    
+    tv.tv_sec = 0;
+    tv.tv_usec = 0;
+    
+    FD_ZERO(&fds);
+    FD_SET(STDIN_FILENO, &fds);
+    
+    ret = select(STDIN_FILENO + 1, &fds, NULL, NULL, &tv);
+    
+    if (ret > 0 && FD_ISSET(STDIN_FILENO, &fds))
+        return 1;
+    
+    return 0;
+}
+#endif
 
 static void process_echoback(unsigned long i)
 {
     unsigned long msg = i >> 4;
-    unsigned char cmd;
     static int _pos = -1;
     static int tpo, bpm, nch, _tpo = -1, _bpm = -1;
     static int pos, pat;
-    static int pause = 0;
-    int k;
 
 #ifdef SIGUSR1
     if (sigusr == SIGUSR1) {
 	skip = 1;
 	xmp_mod_stop(ctx);
-	if (pause)
-	    pause = xmp_mod_pause(ctx);
+	paused = 0;
 	sigusr = 0;
     } else if (sigusr == SIGUSR2) {
 	skip = -1;
 	xmp_mod_stop(ctx);
-	if (pause)
-	    pause = xmp_mod_pause(ctx);
+	paused = 0;
 	sigusr = 0;
     }
 #endif
@@ -237,6 +280,21 @@ static void process_echoback(unsigned long i)
     if (background)
 	return;
 
+    if (showtime) {
+	switch (i & 0xf) {
+	case XMP_ECHO_TIME:
+	    if (1 || showtime) {
+		int rem = mi.time / 100 - msg;
+		fprintf(stderr, "\r%02d:%02d:%02d.%d  ",
+			(int)(msg / (60 * 600)), (int)((msg / 600) % 60),
+			(int)((msg / 10) % 60), (int)(msg % 10));
+		fprintf(stderr, "-%02d:%02d:%02d.%d",
+			(int)(rem / (60 * 600)), (int)((rem / 600) % 60),
+			(int)((rem / 10) % 60), (int)(rem % 10));
+	    }
+	    break;
+	}
+    }
     if (verbosity) {
 	switch (i & 0xf) {
 	case XMP_ECHO_BPM:
@@ -255,6 +313,8 @@ static void process_echoback(unsigned long i)
 		max_nch = nch;
 	    break;
 	case XMP_ECHO_ROW:
+	    if (showtime)
+		break;
 	    rows += 1;
 	    tot_nch += nch;
 	    if
@@ -271,6 +331,13 @@ static void process_echoback(unsigned long i)
 	    break;
 	}
     }
+}
+
+
+void read_keyboard()
+{
+    unsigned char cmd;
+    int k;
 
     /* Interactive commands */
 
@@ -278,6 +345,10 @@ static void process_echoback(unsigned long i)
 	return;
 
 #if defined HAVE_TERMIOS_H && !defined WIN32
+#ifdef __CYGWIN__
+    k = 0;
+    if (stdin_ready_for_reading())
+#endif
     k = read(0, &cmd, 1);
 #elif defined WIN32
     k = cmd = kbhit() ? getch() : 0;
@@ -301,35 +372,33 @@ static void process_echoback(unsigned long i)
 	case 'q':	/* quit */
 	    skip = -2;
 	    xmp_mod_stop(ctx);
-	    if (pause)
-		pause = xmp_mod_pause(ctx);
+	    paused = 0;
 	    break;
 	case 'f':	/* jump to next order */
 	    xmp_ord_next(ctx);
-	    if (pause)
-		pause = xmp_mod_pause(ctx);
+	    paused = 0;
 	    break;
 	case 'b':	/* jump to previous order */
 	    xmp_ord_prev(ctx);
-	    if (pause)
-		pause = xmp_mod_pause(ctx);
+	    paused = 0;
 	    break;
 	case 'n':	/* skip to next module */
 	    skip = 1;
 	    xmp_mod_stop(ctx);
-	    if (pause)
-		pause = xmp_mod_pause(ctx);
+	    paused = 0;
 	    break;
 	case 'p':	/* skip to previous module */
 	    skip = -1;
 	    xmp_mod_stop(ctx);
-	    if (pause)
-		pause = xmp_mod_pause(ctx);
+	    paused = 0;
 	    break;
-	case ' ':	/* pause module */
-	    fprintf (stderr, "%s",  (pause = xmp_mod_pause(ctx))
-		? "] - PAUSED\b\b\b\b\b\b\b\b\b\b"
-		: "]         \b\b\b\b\b\b\b\b\b\b");
+	case ' ':	/* paused module */
+	    paused ^= 1;
+	    if (verbosity) {
+	    	fprintf (stderr, "%s",  paused ?
+				"] - PAUSED\b\b\b\b\b\b\b\b\b\b" :
+				"]         \b\b\b\b\b\b\b\b\b\b");
+	    }
 	    break;
 	case '1':
 	case '2':
@@ -367,10 +436,10 @@ static void shuffle (int argc, char **argv)
 }
 
 
-int main (int argc, char **argv)
+int main(int argc, char **argv)
 {
     int i, t, lf_flag, first, num_mod, verb = 0;
-    time_t t0, t1;
+    time_t t0, t1, t2, t3;
 #ifndef WIN32
     struct timeval tv;
     struct timezone tz;
@@ -384,6 +453,8 @@ int main (int argc, char **argv)
     long rc;
 #endif
 
+    init_drivers();
+
     ctx = xmp_create_context();
 
     xmp_init(ctx, argc, argv);
@@ -444,15 +515,11 @@ int main (int argc, char **argv)
 	}
     }
 
-    xmp_register_event_callback(process_echoback);
+    xmp_register_event_callback(ctx, process_echoback);
 
     if (opt->verbosity) {
-	fprintf (stderr, "Extended Module Player %s %s\n"
-	"Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr\n",
-	    xmp_version, xmp_date);
-#ifdef __EMX__
-	fprintf(stderr, "OS/2 Port by Kevin Langman (langman@earthling.net)\n" );
-#endif
+	fprintf(stderr, "Extended Module Player " VERSION "\n"
+	"Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr\n");
     }
 
     if (probeonly || (opt->verbosity)) {
@@ -512,6 +579,7 @@ int main (int argc, char **argv)
 
     set_tty ();
 
+    paused = 0;
     time (&t0);
 
     num_mod = lf_flag = 0;
@@ -569,10 +637,29 @@ int main (int argc, char **argv)
 	if (loadonly)
 	    goto skip_play;
 
-	t = xmp_play_module(ctx);
+	/* Play the module */
+	time(&t2);
+	xmp_player_start(ctx);
+	for (;;) {
+		read_keyboard();
+
+		if (paused) {
+			usleep(100000);
+		} else {
+			if (xmp_player_frame(ctx) != 0)
+				break;
+			xmp_play_buffer(ctx);
+		}
+	}
+	xmp_player_end(ctx);
+	time(&t3);
+	t = difftime(t3, t2);
 
 	xmp_release_module(ctx);
 
+	if (showtime) {
+	    fprintf(stderr, "\r                       \r");
+	}
 	if (opt->verbosity && !background) {
 	    fprintf (stderr,
 "\rElapsed time   : %dmin%02ds %s                                \n",
