@@ -28,6 +28,7 @@ extern int loadonly;
 extern int randomize;
 extern int nocmd;
 extern int checksum;
+extern int showtime;
 #ifdef HAVE_SYS_RTPRIO_H
 extern int rt;
 #endif
@@ -47,6 +48,7 @@ extern int rt;
 #define OPT_CRUNCH	0x10e
 #define OPT_NOFILTER	0x10f
 #define OPT_VBLANK	0x110
+#define OPT_SHOWTIME	0x111
 
 
 static void exclude_formats(char *list)
@@ -100,13 +102,13 @@ static void list_wrap(char *s, int l, int r, int v)
 
 static void copyright_header()
 {
-    printf("Extended Module Player %s %s\n", xmp_version, xmp_date);
+    printf("Extended Module Player %s\n", VERSION);
     printf(
 "Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr\n"
 "Portions Copyright (C) 1996-1997 Takashi Iwai, (C) 1988 Tammo Hinrichs,\n"
 "(C) 1989 Rich Gopstein and Harris Corporation, (C) 1997 Bert Jahn, (C) 1998\n"
 "Sylvain Chipaux, (C) 1998,2000 Olivier Lapicque, (C) 1999 Tatsuyuki Satoh,\n"
-"(C) 2001-2006 Russell Marks\n\n");
+"(C) 2001-2006 Russell Marks, (C) 2005-2006 Michael Kohn\n\n");
 }
 
 
@@ -118,8 +120,6 @@ static void usage(char *s, struct xmp_options *opt)
     int i;
 
     copyright_header ();
-    if (xmp_build)
-	printf ("%s\n", xmp_build); 
 
     printf ("Usage: %s [options] [modules]\n", s);
 
@@ -162,7 +162,6 @@ static void usage(char *s, struct xmp_options *opt)
 "   --offset-bug-emulation Emulate Protracker 2.x bug in effect 9\n"
 "   -l --loop              Enable module looping\n"
 "   -M --mute ch-list      Mute the specified channels\n"
-"   --modrange             Limit the octave range to 3 octaves in MOD files\n"
 "   --nocmd                Disable interactive commands\n"
 "   --norc                 Don't read configuration files\n"
 "   -R --random            Random order playing\n"
@@ -202,6 +201,7 @@ static void usage(char *s, struct xmp_options *opt)
 "   --load-only            Load module and exit\n"
 "   --probe-only           Probe audio device and exit\n"
 "   -q --quiet             Quiet mode (verbosity level = 0)\n"
+"   --show-time            Display elapsed and remaining time\n"
 "   -V --version           Print version information\n"
 "   -v --verbose           Verbose mode (incremental)\n"
 	, opt->mix,
@@ -247,6 +247,7 @@ void get_options(int argc, char **argv, struct xmp_options *opt, xmp_context ctx
 #endif
 	{ "reverb",		 1, 0, OPT_CHORUS },
 	{ "reverse",		 0, 0, 'r' },
+	{ "show-time",		 0, 0, OPT_SHOWTIME },
 	{ "solo",		 1, 0, 'S' },
 	{ "start",		 1, 0, 's' },
 	{ "stdout",		 0, 0, 'c' },
@@ -294,7 +295,7 @@ void get_options(int argc, char **argv, struct xmp_options *opt, xmp_context ctx
 	    opt->flags |= XMP_CTL_FIXLOOP;
 	    break;
 	case OPT_FX9BUG:
-	    opt->flags |= XMP_CTL_FX9BUG;
+	    opt->quirk |= XMP_QRK_FX9BUG;
 	    break;
 	case 'F':
 	    opt->flags |= XMP_CTL_FILTER;
@@ -391,6 +392,9 @@ void get_options(int argc, char **argv, struct xmp_options *opt, xmp_context ctx
 	case 's':
 	    opt->start = strtoul(optarg, NULL, 0);
 	    break;
+	case OPT_SHOWTIME:
+	    showtime = 1;
+	    break;
 	case OPT_STEREO:
 	    opt->outfmt &= ~XMP_FMT_MONO;
 	    break;
@@ -407,8 +411,8 @@ void get_options(int argc, char **argv, struct xmp_options *opt, xmp_context ctx
 	    opt->flags |= XMP_CTL_VBLANK;
 	    break;
 	case 'V':
-	    printf ("Extended Module Player %s\n", xmp_version);
-	    exit (0);
+	    puts("Extended Module Player " VERSION);
+	    exit(0);
 	case 'v':
 	    opt->verbosity++;
 	    break;
