@@ -22,7 +22,6 @@
 #include "mixer.h"
 
 static int drv_parm = 0;
-extern struct xmp_drv_info drv_callback;
 
 
 int pw_init(void);
@@ -51,9 +50,6 @@ void *xmp_create_context()
 	o->flags = XMP_CTL_DYNPAN | XMP_CTL_FILTER | XMP_CTL_ITPT;
 	o->cf_cutoff = 0;
 
-	/* Set max number of voices per channel */
-	o->maxvoc = 16;
-
 	return ctx;
 }
 
@@ -67,28 +63,15 @@ struct xmp_options *xmp_get_options(xmp_context ctx)
 	return &((struct xmp_context *)ctx)->o;
 }
 
-void xmp_init_callback(xmp_context ctx, void (*callback) (void *, int))
-{
-	struct xmp_options *o = &((struct xmp_context *)ctx)->o;
-
-	xmp_drv_register(&drv_callback);
-	xmp_init_formats(ctx);
-	pw_init();
-
-	xmp_register_driver_callback(ctx, callback);
-
-	o->drv_id = "callback";
-}
-
 void xmp_init(xmp_context ctx, int argc, char **argv)
 {
+	struct xmp_player_context *p = &((struct xmp_context *)ctx)->p;
 	int num;
 
-	xmp_init_drivers();
 	xmp_init_formats(ctx);
 	pw_init();
 
-	xmp_event_callback = NULL;
+	p->event_callback = NULL;
 
 	/* must be parsed before loading the rc file. */
 	for (num = 1; num < argc; num++) {
@@ -97,8 +80,6 @@ void xmp_init(xmp_context ctx, int argc, char **argv)
 	}
 	if (num >= argc)
 		_xmp_read_rc((struct xmp_context *)ctx);
-
-	/* _xmp_tell_wait(); */
 }
 
 inline int xmp_open_audio(xmp_context ctx)
@@ -119,9 +100,11 @@ void xmp_set_driver_parameter(struct xmp_options *o, char *s)
 	drv_parm++;
 }
 
-inline void xmp_register_event_callback(void (*cb) (unsigned long))
+inline void xmp_register_event_callback(xmp_context ctx, void (*cb) (unsigned long))
 {
-	xmp_event_callback = cb;
+	struct xmp_player_context *p = &((struct xmp_context *)ctx)->p;
+
+	p->event_callback = cb;
 }
 
 void xmp_channel_mute(xmp_context ctx, int from, int num, int on)
@@ -155,9 +138,6 @@ int xmp_player_ctl(xmp_context ctx, int cmd, int arg)
 	case XMP_MOD_STOP:
 		p->pos = -2;
 		break;
-	case XMP_MOD_PAUSE:
-		p->pause ^= 1;
-		return p->pause;
 	case XMP_MOD_RESTART:
 		p->pos = -1;
 		break;
@@ -180,43 +160,31 @@ int xmp_player_ctl(xmp_context ctx, int cmd, int arg)
 	return 0;
 }
 
-int xmp_play_module(xmp_context ctx)
+inline int xmp_player_start(xmp_context ctx)
 {
-	struct xmp_options *o = &((struct xmp_context *)ctx)->o;
-	time_t t0, t1;
-	int t;
-
-	time(&t0);
-	_xmp_player_start((struct xmp_context *)ctx);
-	time(&t1);
-	t = difftime(t1, t0);
-
-	o->start = 0;
-
-	return t;
+	return _xmp_player_start((struct xmp_context *)ctx);
 }
 
-#ifdef TEST_OPEN_LOOP
-
-int xmp_play_module2(xmp_context ctx)
+inline int xmp_player_frame(xmp_context ctx)
 {
-	struct xmp_options *o = &((struct xmp_context *)ctx)->o;
-	time_t t0, t1;
-	int t;
-
-	time(&t0);
-	xmp_player_start((struct xmp_context *)ctx);
-	while (xmp_player_loop((struct xmp_context *)ctx) == 0);
-	xmp_player_end((struct xmp_context *)ctx);
-	time(&t1);
-	t = difftime(t1, t0);
-
-	o->start = 0;
-
-	return t;
+	return _xmp_player_frame((struct xmp_context *)ctx);
 }
-#endif
 
+inline void xmp_player_end(xmp_context ctx)
+{
+	_xmp_player_end((struct xmp_context *)ctx);
+}
+
+inline void xmp_play_buffer(xmp_context ctx)
+{
+	xmp_drv_bufdump((struct xmp_context *)ctx);
+}
+
+void xmp_get_buffer(xmp_context ctx, void **buffer, int *size)
+{
+	*size = xmp_smix_softmixer((struct xmp_context *)ctx);
+	*buffer = xmp_smix_buffer((struct xmp_context *)ctx);
+}
 
 void xmp_get_driver_cfg(xmp_context ctx, int *srate, int *res, int *chn,
 			int *itpt)
