@@ -47,6 +47,8 @@ int decrunch_arcfs	(FILE *, FILE *);
 int decrunch_sqsh	(FILE *, FILE *);
 int decrunch_pp		(FILE *, FILE *);
 int decrunch_mmcmp	(FILE *, FILE *);
+int decrunch_muse	(FILE *, FILE *);
+int decrunch_lzx	(FILE *, FILE *);
 int decrunch_oxm	(FILE *, FILE *);
 int decrunch_xfd	(FILE *, FILE *);
 int test_oxm		(FILE *);
@@ -60,11 +62,13 @@ char *test_xfd		(unsigned char *, int);
 #define BUILTIN_S404	0x07
 #define BUILTIN_OXM	0x08
 #define BUILTIN_XFD	0x09
+#define BUILTIN_MUSE	0x0a
+#define BUILTIN_LZX	0x0b
 
 
 #if defined __EMX__ || defined WIN32
 #define REDIR_STDERR "2>NUL"
-#elif defined unix
+#elif defined unix || defined __unix__
 #define REDIR_STDERR "2>/dev/null"
 #else
 #define REDIR_STDERR
@@ -109,8 +113,9 @@ static int decrunch(struct xmp_context *ctx, FILE **f, char **s)
 	fprintf( stderr, "LHA for OS/2 does NOT support output to stdout.\n" );
 #elif defined __AMIGA__
 	cmd = "lha p -q \"%s\"";
-#endif
+#else
 	cmd = "lha -pq \"%s\"";
+#endif
     } else if (b[0] == 31 && b[1] == 139) {
 	packer = "gzip";
 	cmd = "gzip -dc \"%s\"";
@@ -120,6 +125,9 @@ static int decrunch(struct xmp_context *ctx, FILE **f, char **s)
     } else if (b[0] == 0x5d && b[1] == 0 && b[2] == 0 && b[3] == 0x80) {
 	packer = "lzma";
 	cmd = "lzma -dc \"%s\"";
+    } else if (b[0] == 0xfd && b[3] == 'X' && b[4] == 'Z' && b[5] == 0x00) {
+	packer = "xz";
+	cmd = "xz -dc \"%s\"";
     } else if (b[0] == 'Z' && b[1] == 'O' && b[2] == 'O' && b[3] == ' ') {
 	packer = "zoo";
 	cmd = "zoo xpq \"%s\"";
@@ -147,6 +155,17 @@ static int decrunch(struct xmp_context *ctx, FILE **f, char **s)
 		b[4] == 'O' && b[5] == 'N' && b[6] == 'i' && b[7] == 'a') {
 	packer = "MMCMP";
 	builtin = BUILTIN_MMCMP;
+    } else if (b[0] == 'M' && b[1] == 'U' && b[2] == 'S' && b[3] == 'E' &&
+		b[4] == 0xde && b[5] == 0xad && b[6] == 0xbe && b[7] == 0xaf) {
+	packer = "J2B MUSE";
+	builtin = BUILTIN_MUSE;
+    } else if (b[0] == 'M' && b[1] == 'U' && b[2] == 'S' && b[3] == 'E' &&
+		b[4] == 0xde && b[5] == 0xad && b[6] == 0xba && b[7] == 0xbe) {
+	packer = "MOD2J2B MUSE";
+	builtin = BUILTIN_MUSE;
+    } else if (b[0] == 76 && b[1] == 90 && b[2] == 88) {
+	packer = "LZX";
+	builtin = BUILTIN_LZX;
     } else if (b[0] == 'R' && b[1] == 'a' && b[2] == 'r') {
 	packer = "rar";
 	cmd = "unrar p -inul -xreadme -x*.diz -x*.nfo -x*.txt "
@@ -275,6 +294,12 @@ static int decrunch(struct xmp_context *ctx, FILE **f, char **s)
 	case BUILTIN_MMCMP:    
 	    res = decrunch_mmcmp(*f, t);
 	    break;
+	case BUILTIN_MUSE:    
+	    res = decrunch_muse(*f, t);
+	    break;
+	case BUILTIN_LZX:    
+	    res = decrunch_lzx(*f, t);
+	    break;
 #if !defined WIN32 && !defined __MINGW32__ && !defined __AMIGA__
 	case BUILTIN_OXM:
 	    res = decrunch_oxm(*f, t);
@@ -507,7 +532,8 @@ int xmp_load_module(xmp_context ctx, char *s)
     m->volume = 0x40;
     m->vol_xlat = NULL;
     /* Reset control for next module */
-    m->fetch = o->flags & ~XMP_CTL_FILTER;
+    m->flags = o->flags & ~XMP_CTL_FILTER;	/* verify this later */
+    m->quirk = o->quirk;
     m->comment = NULL;
 
     m->xxh = calloc(sizeof (struct xxm_header), 1);
@@ -580,7 +606,7 @@ int xmp_load_module(xmp_context ctx, char *s)
 	m->xxh->rst = 0;
 
     /* Disable filter if --nofilter is specified */
-    m->fetch &= ~(~o->flags & XMP_CTL_FILTER);
+    m->flags &= ~(~o->flags & XMP_CTL_FILTER);
 
     str_adj(m->name);
     if (!*m->name)
@@ -588,7 +614,7 @@ int xmp_load_module(xmp_context ctx, char *s)
 
     if (o->verbosity > 1) {
 	report("Module looping : %s\n",
-	    m->fetch & XMP_CTL_LOOP ? "yes" : "no");
+	    m->flags & XMP_CTL_LOOP ? "yes" : "no");
 	report("Period mode    : %s\n",
 	    m->xxh->flg & XXM_FLG_LINEAR ? "linear" : "Amiga");
     }
@@ -601,8 +627,8 @@ int xmp_load_module(xmp_context ctx, char *s)
 	report("Base volume    : %d\n", m->volbase);
 	report("C4 replay rate : %d\n", m->c4rate);
 	report("Channel mixing : %d%% (dynamic pan %s)\n",
-		m->fetch & XMP_CTL_REVERSE ? -o->mix : o->mix,
-		m->fetch & XMP_CTL_DYNPAN ? "enabled" : "disabled");
+		m->flags & XMP_CTL_REVERSE ? -o->mix : o->mix,
+		m->flags & XMP_CTL_DYNPAN ? "enabled" : "disabled");
 	report("Checksum       : %u %ld\n", crc, st.st_size);
 	report("Volume amplify : %s\n", amp_factor[o->amplify]);
     }
@@ -623,7 +649,7 @@ int xmp_load_module(xmp_context ctx, char *s)
     t = _xmp_scan_module((struct xmp_context *)ctx);
 
     if (o->verbosity) {
-	if (m->fetch & XMP_CTL_LOOP)
+	if (m->flags & XMP_CTL_LOOP)
 	    report ("One loop time  : %dmin%02ds\n",
 		(t + 500) / 60000, ((t + 500) / 1000) % 60);
 	else
