@@ -1,8 +1,6 @@
 /* Extended Module Player OMX depacker
  * Copyright (C) 2007 Claudio Matsuoka
  *
- * $Id: oxm.c,v 1.10 2007/12/02 01:19:45 cmatsuoka Exp $
- *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
  * for more information.
@@ -22,7 +20,7 @@
 #include <sys/wait.h>
 #include <unistd.h>
 
-#include "xmpi.h"
+#include "common.h"
 
 #define MAGIC_OGGS	0x4f676753
 
@@ -49,7 +47,8 @@ int test_oxm(FILE *f)
 {
 	int i, j;
 	int hlen, npat, len, plen;
-	int nins, nsmp, ilen;
+	int nins, nsmp;
+	uint32 ilen;
 	int slen[256];
 	uint8 buf[1024];
 
@@ -209,7 +208,8 @@ int decrunch_oxm(FILE *f, FILE *fo)
 {
 	int i, j, pos;
 	int hlen, npat, len, plen;
-	int nins, nsmp, ilen;
+	int nins, nsmp;
+	uint32 ilen;
 	uint8 buf[1024];
 	struct xm_instrument xi[256];
 	char *pcm[256];
