@@ -17,128 +17,14 @@
 #include "mixer.h"
 #include "period.h"
 #include "readlzw.h"
+#include "synth.h"
+#include "smix.h"
 
 #define	FREE	-1
+#define MAX_VOICES_CHANNEL 16
 
 static struct xmp_drv_info *drv_array = NULL;
 
-static int numvoc;		/* number of soundcard voices */
-static int numchn;		/* number of channels need to play one module */
-static int numtrk;		/* number of tracks in module */
-static int maxvoc;		/* */
-static int agevoc;		/* */
-static int extern_drv;		/* set output to driver other than mixer */
-
-static inline void drv_resetvoice (struct xmp_context *, int, int);
-
-
-extern struct xmp_drv_info drv_file;
-extern struct xmp_drv_info drv_wav;
-
-extern struct xmp_drv_info drv_osx;
-extern struct xmp_drv_info drv_solaris;
-extern struct xmp_drv_info drv_hpux;
-extern struct xmp_drv_info drv_bsd;
-extern struct xmp_drv_info drv_netbsd;
-extern struct xmp_drv_info drv_openbsd;
-extern struct xmp_drv_info drv_sgi;
-extern struct xmp_drv_info drv_aix;
-extern struct xmp_drv_info drv_oss_seq;
-extern struct xmp_drv_info drv_oss;
-extern struct xmp_drv_info drv_alsa;
-extern struct xmp_drv_info drv_alsa05;
-extern struct xmp_drv_info drv_net;
-extern struct xmp_drv_info drv_esd;
-extern struct xmp_drv_info drv_arts;
-extern struct xmp_drv_info drv_nas;
-extern struct xmp_drv_info drv_pulseaudio;
-extern struct xmp_drv_info drv_os2dart;
-extern struct xmp_drv_info drv_qnx;
-extern struct xmp_drv_info drv_beos;
-extern struct xmp_drv_info drv_amiga;
-extern struct xmp_drv_info drv_win32;
-
-
-void (*xmp_event_callback) (unsigned long);
-
-#include "../player/mixer.c"
-
-
-void xmp_init_drivers()
-{
-#ifndef ENABLE_PLUGIN
-
-    /* Output to file will be always available -- except in the plugin ;) */
-    xmp_drv_register(&drv_file);
-    xmp_drv_register(&drv_wav);
-
-#ifdef DRIVER_OSX
-    xmp_drv_register(&drv_osx);
-#endif
-#ifdef DRIVER_WIN32
-    xmp_drv_register(&drv_win32);
-#endif
-#ifdef DRIVER_SOLARIS
-    xmp_drv_register(&drv_solaris);
-#endif
-#ifdef DRIVER_HPUX
-    xmp_drv_register(&drv_hpux);
-#endif
-#ifdef DRIVER_BSD
-    xmp_drv_register(&drv_bsd);
-#endif
-#ifdef DRIVER_NETBSD
-    xmp_drv_register(&drv_netbsd);
-#endif
-#ifdef DRIVER_OPENBSD
-    xmp_drv_register(&drv_openbsd);
-#endif
-#ifdef DRIVER_SGI
-    xmp_drv_register(&drv_sgi);
-#endif
-#ifdef DRIVER_OSS_SEQ
-    xmp_drv_register(&drv_oss_seq);
-#endif
-#ifdef DRIVER_OSS
-    xmp_drv_register(&drv_oss);
-#endif
-#ifdef DRIVER_ALSA
-    xmp_drv_register(&drv_alsa);
-#endif
-#ifdef DRIVER_ALSA05
-    xmp_drv_register(&drv_alsa05);
-#endif
-#ifdef DRIVER_QNX
-    xmp_drv_register(&drv_qnx);
-#endif
-#ifdef DRIVER_BEOS
-    xmp_drv_register(&drv_beos);
-#endif
-#ifdef DRIVER_AMIGA
-    xmp_drv_register(&drv_amiga);
-#endif
-#ifdef DRIVER_NET
-    xmp_drv_register(&drv_net);
-#endif
-#ifdef DRIVER_OS2DART
-    xmp_drv_register(&drv_os2dart);
-#endif
-#ifdef DRIVER_PULSEAUDIO
-    xmp_drv_register(&drv_pulseaudio);
-#endif
-#ifdef DRIVER_ARTS
-    xmp_drv_register(&drv_arts);
-#endif
-#ifdef DRIVER_ESD
-    xmp_drv_register(&drv_esd);
-#endif
-#ifdef DRIVER_NAS
-    xmp_drv_register(&drv_nas);
-#endif
-
-#endif	/* ENABLE_PLUGIN */
-}
-
 
 static int drv_select(struct xmp_context *ctx)
 {
@@ -160,9 +46,8 @@ static int drv_select(struct xmp_context *ctx)
 	}
     } else {
 	ret = XMP_ERR_DSPEC;
-	drv = drv_array->next;	/* skip file */
-	drv = drv->next;	/* skip wav */
-	for (; drv; drv = drv->next) {
+
+	for (drv = drv_array; drv; drv = drv->next) {
 	    if (o->verbosity > 2)
 		report ("Probing %s... ", drv->description);
 	    if (drv->init(ctx) == 0) {
@@ -187,18 +72,18 @@ static int drv_select(struct xmp_context *ctx)
 }
 
 
-static void drv_resetvoice(struct xmp_context *ctx, int voc, int mute)
+void xmp_drv_resetvoice(struct xmp_context *ctx, int voc, int mute)
 {
     struct xmp_driver_context *d = &ctx->d;
     struct voice_info *vi = &d->voice_array[voc];
 
-    if ((uint32) voc >= numvoc)
+    if ((uint32)voc >= d->maxvoc)
 	return;
 
     if (mute)
 	d->driver->setvol(ctx, voc, 0);
 
-    d->numvoc--;
+    d->curvoc--;
     d->ch2vo_count[vi->root]--;
     d->ch2vo_array[vi->chn] = FREE;
     memset(vi, 0, sizeof (struct voice_info));
@@ -224,18 +109,19 @@ int xmp_drv_open(struct xmp_context *ctx)
 {
     int status;
     struct xmp_driver_context *d = &ctx->d;
+    struct xmp_smixer_context *s = &ctx->s;
     struct xmp_options *o = &ctx->o;
 
     d->memavl = 0;
-    smix_buf32b = NULL;
-    extern_drv = TURN_ON;
+    s->buf32b = NULL;
+    d->ext = 1;
     if ((status = drv_select(ctx)) != 0)
 	return status;
 
     d->patch_array = calloc(XMP_MAXPAT, sizeof(struct patch_info *));
 
     if (d->patch_array == NULL) {
-	d->driver->shutdown ();
+	d->driver->shutdown(ctx);
 	return XMP_ERR_ALLOC;
     }
 
@@ -273,9 +159,9 @@ void xmp_drv_close(struct xmp_context *ctx)
 
     xmp_drv_off(ctx);
     memset(d->cmute_array, 0, XMP_MAXCH * sizeof(int));
+    d->driver->shutdown(ctx);
     free(d->patch_array);
-    d->driver->shutdown();
-    synth_deinit();
+    /*synth_deinit();*/
 }
 
 
@@ -284,38 +170,38 @@ int xmp_drv_on(struct xmp_context *ctx, int num)
 {
     struct xmp_player_context *p = &ctx->p;
     struct xmp_driver_context *d = &ctx->d;
+    struct xmp_smixer_context *s = &ctx->s;
     struct xmp_mod_context *m = &p->m;
     struct xmp_options *o = &ctx->o;
 
-    numtrk = d->numtrk = num;
-    num = d->driver->numvoices(135711);
+    d->numtrk = num;
+    num = d->driver->numvoices(ctx, 135711);
     d->driver->reset();
 
-    numchn = numtrk;
-    maxvoc = m->fetch & XMP_CTL_VIRTUAL ? o->maxvoc : 1;
+    d->numchn = d->numtrk;
+    d->chnvoc = m->flags & XMP_CTL_VIRTUAL ? MAX_VOICES_CHANNEL : 1;
 
-    if (maxvoc > 1)
-	numchn += num;
-    else if (num > numchn)
-	num = numchn;
+    if (d->chnvoc > 1)
+	d->numchn += num;
+    else if (num > d->numchn)
+	num = d->numchn;
 
-    num = numvoc = d->driver->numvoices(num);
+    num = d->maxvoc = d->driver->numvoices(ctx, num);
 
-    d->voice_array = calloc(numvoc, sizeof (struct voice_info));
-    d->ch2vo_array = calloc(numchn, sizeof (int));
-    d->ch2vo_count = calloc(numchn, sizeof (int));
+    d->voice_array = calloc(d->maxvoc, sizeof (struct voice_info));
+    d->ch2vo_array = calloc(d->numchn, sizeof (int));
+    d->ch2vo_count = calloc(d->numchn, sizeof (int));
 
     if (!(d->voice_array && d->ch2vo_array && d->ch2vo_count))
 	return XMP_ERR_ALLOC;
 
     for (; num--; d->voice_array[num].chn = d->voice_array[num].root = FREE);
-    for (num = numchn; num--; d->ch2vo_array[num] = FREE);
+    for (num = d->numchn; num--; d->ch2vo_array[num] = FREE);
 
-    d->numvoc = agevoc = 0;
-    d->numchn = numchn;
+    d->curvoc = d->agevoc = 0;
 
-    smix_mode = o->outfmt & XMP_FMT_MONO ? 1 : 2;
-    smix_resol = o->resol > 8 ? 2 : 1;
+    s->mode = o->outfmt & XMP_FMT_MONO ? 1 : 2;
+    s->resol = o->resol > 8 ? 2 : 1;
     smix_resetvar(ctx);
 
     return 0;
@@ -326,13 +212,13 @@ void xmp_drv_off(struct xmp_context *ctx)
 {
     struct xmp_driver_context *d = &ctx->d;
 
-    if (numchn < 1)
+    if (d->numchn < 1)
 	return;
 
     xmp_drv_writepatch(ctx, NULL);
-    d->numvoc = numvoc = 0;
-    d->numchn = numchn = 0;
-    numtrk = 0;
+    d->curvoc = d->maxvoc = 0;
+    d->numchn = 0;
+    d->numtrk = 0;
     free(d->voice_array);
     free(d->ch2vo_array);
     free(d->ch2vo_count);
@@ -351,22 +237,25 @@ void xmp_drv_off(struct xmp_context *ctx)
 void xmp_drv_reset(struct xmp_context *ctx)
 {
     struct xmp_driver_context *d = &ctx->d;
-    int num;
+    int i;
 
-    if (numchn < 1)
+    if (d->numchn < 1)
 	return;
 
-    d->driver->numvoices(d->driver->numvoices (43210));
+    d->driver->numvoices(ctx, d->driver->numvoices(ctx, 43210));
     d->driver->reset();
-    d->driver->numvoices(num = numvoc);
+    d->driver->numvoices(ctx, d->maxvoc);
 
-    memset(d->ch2vo_count, 0, numchn * sizeof (int));
-    memset(d->voice_array, 0, numvoc * sizeof (struct voice_info));
+    memset(d->ch2vo_count, 0, d->numchn * sizeof (int));
+    memset(d->voice_array, 0, d->maxvoc * sizeof (struct voice_info));
 
-    for (; num--; d->voice_array[num].chn = d->voice_array[num].root = FREE);
-    for (num = numchn; num--; d->ch2vo_array[num] = FREE);
+    for (i = 0; i < d->maxvoc; i++)
+	 d->voice_array[i].chn = d->voice_array[i].root = FREE;
 
-    d->numvoc = agevoc = 0;
+    for (i = 0; i < d->numchn; i++)
+	d->ch2vo_array[i] = FREE;
+
+    d->curvoc = d->agevoc = 0;
 }
 
 
@@ -377,12 +266,12 @@ void xmp_drv_resetchannel(struct xmp_context *ctx, int chn)
 
     voc = d->ch2vo_array[chn];
 
-    if ((uint32)chn >= numchn || (uint32)voc >= numvoc)
+    if ((uint32)chn >= d->numchn || (uint32)voc >= d->maxvoc)
 	return;
 
     d->driver->setvol(ctx, voc, 0);
 
-    d->numvoc--;
+    d->curvoc--;
     d->ch2vo_count[d->voice_array[voc].root]--;
     d->ch2vo_array[chn] = FREE;
     memset(&d->voice_array[voc], 0, sizeof (struct voice_info));
@@ -396,25 +285,25 @@ static int drv_allocvoice(struct xmp_context *ctx, int chn)
     int voc, vfree;
     uint32 age;
 
-    if (d->ch2vo_count[chn] < maxvoc) {
-	for (voc = numvoc; voc-- && d->voice_array[voc].chn != FREE;);
+    if (d->ch2vo_count[chn] < d->chnvoc) {
+	for (voc = d->maxvoc; voc-- && d->voice_array[voc].chn != FREE;);
 	if (voc < 0)
 	    return voc;
 
-	d->voice_array[voc].age = agevoc;
+	d->voice_array[voc].age = d->agevoc;
 	d->ch2vo_count[chn]++;
-	d->numvoc++;
+	d->curvoc++;
 
 	return voc;
     }
 
-    for (voc = numvoc, vfree = age = FREE; voc--;) {
+    for (voc = d->maxvoc, vfree = age = FREE; voc--;) {
 	if (d->voice_array[voc].root == chn && d->voice_array[voc].age < age)
 	    age = d->voice_array[vfree = voc].age;
     }
 
     d->ch2vo_array[d->voice_array[vfree].chn] = FREE;
-    d->voice_array[vfree].age = agevoc;
+    d->voice_array[vfree].age = d->agevoc;
 
     return vfree;
 }
@@ -441,16 +330,16 @@ void xmp_drv_setvol(struct xmp_context *ctx, int chn, int vol)
 
     voc = d->ch2vo_array[chn];
 
-    if ((uint32)chn >= numchn || (uint32)voc >= numvoc)
+    if ((uint32)chn >= d->numchn || (uint32)voc >= d->maxvoc)
 	return;
 
     if (d->voice_array[voc].root < XMP_MAXCH && d->cmute_array[d->voice_array[voc].root])
-	vol = TURN_OFF;
+	vol = 0;
 
     d->driver->setvol(ctx, voc, vol);
 
-    if (!(vol || chn < numtrk))
-	drv_resetvoice(ctx, voc, TURN_ON);
+    if (!(vol || chn < d->numtrk))
+	xmp_drv_resetvoice(ctx, voc, 1);
 }
 
 
@@ -461,7 +350,7 @@ void xmp_drv_setpan(struct xmp_context *ctx, int chn, int pan)
 
     voc = d->ch2vo_array[chn];
 
-    if ((uint32)chn >= numchn || (uint32)voc >= numvoc)
+    if ((uint32)chn >= d->numchn || (uint32)voc >= d->maxvoc)
 	return;
 
     d->driver->setpan(ctx, voc, pan);
@@ -475,7 +364,7 @@ void xmp_drv_seteffect(struct xmp_context *ctx, int chn, int type, int val)
 
     voc = d->ch2vo_array[chn];
 
-    if ((uint32)chn >= numchn || (uint32)voc >= numvoc)
+    if ((uint32)chn >= d->numchn || (uint32)voc >= d->maxvoc)
 	return;
 
     d->driver->seteffect(ctx, voc, type, val);
@@ -490,7 +379,7 @@ void xmp_drv_setsmp(struct xmp_context *ctx, int chn, int smp)
 
     voc = d->ch2vo_array[chn];
 
-    if ((uint32)chn >= numchn || (uint32)voc >= numvoc)
+    if ((uint32)chn >= d->numchn || (uint32)voc >= d->maxvoc)
 	return;
 
     vi = &d->voice_array[voc];
@@ -503,7 +392,7 @@ void xmp_drv_setsmp(struct xmp_context *ctx, int chn, int smp)
     smix_setpatch(ctx, voc, smp);
     smix_voicepos(ctx, voc, pos, itp);
 
-    if (extern_drv) {
+    if (d->ext) {
 	d->driver->setpatch(voc, smp);
 	d->driver->setnote(voc, vi->note);
 	d->driver->voicepos(voc,
@@ -517,14 +406,14 @@ int xmp_drv_setpatch(struct xmp_context *ctx, int chn, int ins, int smp, int not
     struct xmp_driver_context *d = &ctx->d;
     int voc, vfree;
 
-    if ((uint32) chn >= numchn)
+    if ((uint32)chn >= d->numchn)
 	return -1;
 
     if (ins < 0 || (uint32)smp >= XMP_MAXPAT || !d->patch_array[smp])
 	smp = -1;
 
     if (dct) {
-	for (voc = numvoc; voc--;) {
+	for (voc = d->maxvoc; voc--;) {
 	    if (d->voice_array[voc].root == chn && d->voice_array[voc].ins == ins) {
 		if ((dct == XXM_DCT_INST) ||
 		    (dct == XXM_DCT_SMP && d->voice_array[voc].smp == smp) ||
@@ -533,7 +422,7 @@ int xmp_drv_setpatch(struct xmp_context *ctx, int chn, int ins, int smp, int not
 			if (voc != d->ch2vo_array[chn] || d->voice_array[voc].act)
 			    d->voice_array[voc].act = dca;
 		    } else {
-			drv_resetvoice(ctx, voc, TURN_ON);
+			xmp_drv_resetvoice(ctx, voc, 1);
 		    }
 		}
 	    }
@@ -543,11 +432,11 @@ int xmp_drv_setpatch(struct xmp_context *ctx, int chn, int ins, int smp, int not
     voc = d->ch2vo_array[chn];
 
     if (voc > FREE) {
-	if (d->voice_array[voc].act && maxvoc > 1) {
+	if (d->voice_array[voc].act && d->chnvoc > 1) {
 	    if ((vfree = drv_allocvoice(ctx, chn)) > FREE) {
 		d->voice_array[vfree].root = chn;
 		d->voice_array[d->ch2vo_array[chn] = vfree].chn = chn;
-		for (chn = numtrk; d->ch2vo_array[chn++] > FREE;);
+		for (chn = d->numtrk; d->ch2vo_array[chn++] > FREE;);
 		d->voice_array[voc].chn = --chn;
 		d->ch2vo_array[chn] = voc;
 		voc = vfree;
@@ -563,7 +452,7 @@ int xmp_drv_setpatch(struct xmp_context *ctx, int chn, int ins, int smp, int not
     }
 
     if (smp < 0) {
-	drv_resetvoice(ctx, voc, TURN_ON);
+	xmp_drv_resetvoice(ctx, voc, 1);
 	return chn;	/* was -1 */
     }
 
@@ -573,13 +462,13 @@ int xmp_drv_setpatch(struct xmp_context *ctx, int chn, int ins, int smp, int not
     d->voice_array[voc].ins = ins;
     d->voice_array[voc].act = nna;
 
-    if (extern_drv) {
+    if (d->ext) {
 	if (!cont_sample)
 	    d->driver->setpatch(voc, smp);
 	d->driver->setnote(voc, note);
     }
 
-    agevoc++;
+    d->agevoc++;
 
     return chn;
 }
@@ -592,7 +481,7 @@ void xmp_drv_setnna(struct xmp_context *ctx, int chn, int nna)
 
     voc = d->ch2vo_array[chn];
 
-    if ((uint32)chn >= numchn || (uint32)voc >= numvoc)
+    if ((uint32)chn >= d->numchn || (uint32)voc >= d->maxvoc)
 	return;
 
     d->voice_array[voc].act = nna;
@@ -606,12 +495,12 @@ void xmp_drv_setbend(struct xmp_context *ctx, int chn, int bend)
 
     voc = d->ch2vo_array[chn];
 
-    if ((uint32)chn >= numchn || (uint32)voc >= numvoc)
+    if ((uint32)chn >= d->numchn || (uint32)voc >= d->maxvoc)
 	return;
 
     smix_setbend(ctx, voc, bend);
 
-    if (extern_drv)
+    if (d->ext)
 	d->driver->setbend(voc, bend);
 }
 
@@ -623,12 +512,12 @@ void xmp_drv_retrig(struct xmp_context *ctx, int chn)
 
     voc = d->ch2vo_array[chn];
 
-    if ((uint32)chn >= numchn || (uint32)voc >= numvoc)
+    if ((uint32)chn >= d->numchn || (uint32)voc >= d->maxvoc)
 	return;
 
     smix_voicepos(ctx, voc, 0, 0);
 
-    if (extern_drv)
+    if (d->ext)
 	d->driver->setnote(voc, d->voice_array[voc].note);
 }
 
@@ -638,10 +527,10 @@ void xmp_drv_pastnote(struct xmp_context *ctx, int chn, int act)
     struct xmp_driver_context *d = &ctx->d;
     int voc;
 
-    for (voc = numvoc; voc--;) {
-	if (d->voice_array[voc].root == chn && d->voice_array[voc].chn >= numtrk) {
+    for (voc = d->maxvoc; voc--;) {
+	if (d->voice_array[voc].root == chn && d->voice_array[voc].chn >= d->numtrk) {
 	    if (act == XMP_ACT_CUT)
-		drv_resetvoice(ctx, voc, TURN_ON);
+		xmp_drv_resetvoice(ctx, voc, 1);
 	    else
 		d->voice_array[voc].act = act;
 	}
@@ -655,7 +544,7 @@ void xmp_drv_voicepos(struct xmp_context *ctx, int chn, int pos)
     int voc;
     struct patch_info *pi;
 
-    if ((uint32) chn >= numchn || (uint32) (voc = d->ch2vo_array[chn]) >= numvoc)
+    if ((uint32)chn >= d->numchn || (uint32) (voc = d->ch2vo_array[chn]) >= d->maxvoc)
 	return;
 
     pi = d->patch_array[d->voice_array[voc].smp];
@@ -669,7 +558,7 @@ void xmp_drv_voicepos(struct xmp_context *ctx, int chn, int pos)
 
     smix_voicepos(ctx, voc, pos, 0);
 
-    if (extern_drv)
+    if (d->ext)
 	d->driver->voicepos(voc, pos << !!(pi->mode & WAVE_16_BITS));
 }
 
@@ -681,10 +570,10 @@ int xmp_drv_cstat(struct xmp_context *ctx, int chn)
 
     voc = d->ch2vo_array[chn];
 
-    if ((uint32) chn >= numchn || (uint32)voc >= numvoc)
+    if ((uint32)chn >= d->numchn || (uint32)voc >= d->maxvoc)
 	return XMP_CHN_DUMB;
 
-    return chn < numtrk ? XMP_CHN_ACTIVE : d->voice_array[voc].act;
+    return chn < d->numtrk ? XMP_CHN_ACTIVE : d->voice_array[voc].act;
 }
 
 
@@ -692,7 +581,7 @@ inline void xmp_drv_echoback(struct xmp_context *ctx, int msg)
 {
     struct xmp_driver_context *d = &ctx->d;
 
-    d->driver->echoback(msg);
+    d->driver->echoback(ctx, msg);
 }
 
 
@@ -700,16 +589,14 @@ inline int xmp_drv_getmsg(struct xmp_context *ctx)
 {
     struct xmp_driver_context *d = &ctx->d;
 
-    return d->driver->getmsg();
+    return d->driver->getmsg(ctx);
 }
 
 
-extern int **xmp_mix_buffer;
-
 inline void xmp_drv_bufdump(struct xmp_context *ctx)
 {
     struct xmp_driver_context *d = &ctx->d;
-    int i = softmixer(ctx);
+    int i = xmp_smix_softmixer(ctx);
 
     d->driver->bufdump(ctx, i);
 }
@@ -729,7 +616,7 @@ void xmp_drv_stoptimer(struct xmp_context *ctx)
     struct xmp_driver_context *d = &ctx->d;
     int voc;
 
-    for (voc = numvoc; voc--; )
+    for (voc = d->maxvoc; voc--; )
 	d->driver->setvol(ctx, voc, 0);
 
     d->driver->stoptimer();
@@ -802,7 +689,7 @@ int xmp_drv_flushpatch(struct xmp_context *ctx, int ratio)
 	if (d->patch_array[smp])
 	    num++;
 
-    if (extern_drv) {
+    if (d->ext) {
 	reportv(ctx, 0, "Uploading smps : %d ", num);
 
 	for (smp = XMP_MAXPAT; smp--;) {
@@ -1000,10 +887,3 @@ inline struct xmp_drv_info *xmp_drv_array()
 {
     return drv_array;
 }
-
-
-void xmp_register_driver_callback(xmp_context ctx, void (*callback)(void *, int))
-{
-    ((struct xmp_context *)ctx)->d.callback = callback;
-}
-
