@@ -6,6 +6,13 @@
  * for more information.
  */
 
+#include "common.h"
+#include "player.h"
+#include "effects.h"
+#include "period.h"
+#include "driver.h"
+#include "mixer.h"
+
 #define NOT_IMPLEMENTED
 
 #define DO_TONEPORTA() { \
@@ -16,18 +23,10 @@
 	xc->s_sgn = xc->period < xc->s_end ? 1 : -1; \
 }
 
+#define HAS_QUIRK(x) (m->quirk & (x))
 
-/* Values for multi-retrig */
-static struct retrig_t rval[] = {
-    {   0,  1,  1 }, {  -1,  1,  1 }, {  -2,  1,  1 }, {  -4,  1,  1 },
-    {  -8,  1,  1 }, { -16,  1,  1 }, {   0,  2,  3 }, {   0,  1,  2 },
-    {   0,  1,  1 }, {   1,  1,  1 }, {   2,  1,  1 }, {   4,  1,  1 },
-    {   8,  1,  1 }, {  16,  1,  1 }, {   0,  3,  2 }, {   0,  2,  1 },
-    {   0,  0,  1 }	/* Note cut */
-};
-    
 
-void process_fx(struct xmp_context *ctx, int chn, uint8 note, uint8 fxt, uint8 fxp, struct xmp_channel *xc)
+void process_fx(struct xmp_context *ctx, int chn, uint8 note, uint8 fxt, uint8 fxp, struct xmp_channel *xc, int fnum)
 {
     struct xmp_player_context *p = &ctx->p;
     struct xmp_mod_context *m = &p->m;
@@ -68,8 +67,10 @@ void process_fx(struct xmp_context *ctx, int chn, uint8 note, uint8 fxt, uint8 f
 	xc->a_size = 3;
 	break;
     case FX_PORTA_UP:				/* Portamento up */
-fx_porta_up:
-	if (m->fetch & XMP_CTL_FINEFX) {
+	if (fxp == 0)
+	    fxp = xc->porta;
+
+	if (HAS_QUIRK(XMP_QRK_FINEFX) && (fnum == 0 || !HAS_QUIRK(XMP_QRK_ITVPOR))) {
 	    switch (MSN(fxp)) {
 	    case 0xf:
 		xc->porta = fxp;
@@ -81,20 +82,21 @@ fx_porta_up:
 		fxp |= 0x10;
 		goto fx_xf_porta;
 	    }
-	    if (!fxp) {
-		if ((fxp = xc->porta) != 0)
-		    goto fx_porta_up;
-	    }
 	}
 	SET(PITCHBEND);
-	if ((xc->porta = fxp) != 0)
+	if ((xc->porta = fxp) != 0) {
 	    xc->f_val = -fxp;
-	else if (xc->f_val > 0)
+	    if (m->quirk & XMP_QRK_UNISLD)
+		xc->s_val = -fxp;
+	} else if (xc->f_val > 0) {
 	    xc->f_val *= -1;
+	}
 	break;
     case FX_PORTA_DN:				/* Portamento down */
-fx_porta_dn:
-	if (m->fetch & XMP_CTL_FINEFX) {
+	if (fxp == 0)
+	    fxp = xc->porta;
+
+	if (HAS_QUIRK(XMP_QRK_FINEFX) && (fnum == 0 || !HAS_QUIRK(XMP_QRK_ITVPOR))) {
 	    switch (MSN(fxp)) {
 	    case 0xf:
 		xc->porta = fxp;
@@ -106,16 +108,15 @@ fx_porta_dn:
 		fxp |= 0x20;
 		goto fx_xf_porta;
 	    }
-	    if (!fxp) {
-		if ((fxp = xc->porta) != 0)
-		    goto fx_porta_dn;
-	    }
 	}
 	SET(PITCHBEND);
-	if ((xc->porta = fxp) != 0)
+	if ((xc->porta = fxp) != 0) {
 	    xc->f_val = fxp;
-	else if (xc->f_val < 0)
+	    if (m->quirk & XMP_QRK_UNISLD)
+		xc->s_val = fxp;
+	} else if (xc->f_val < 0) {
 	    xc->f_val *= -1;
+	}
 	break;
     case FX_TONEPORTA:				/* Tone portamento */
 	if (!TEST (IS_VALID))
@@ -123,7 +124,7 @@ fx_porta_dn:
 	DO_TONEPORTA();
 	if (fxp) {
 	    xc->s_val = fxp;
-	    if (m->fetch & XMP_CTL_UNISLD)	/* IT compatible Gxx off */
+	    if (m->quirk & XMP_QRK_UNISLD)	/* IT compatible Gxx off */
 		xc->porta = fxp;
 	}
 	SET(TONEPORTA);
@@ -232,7 +233,7 @@ fx_porta_dn:
 	 *	DFF command is specified, the volume will be slid up.
 	 */
 fx_volslide:
-	if (m->fetch & XMP_CTL_FINEFX) {
+	if (m->quirk & XMP_QRK_FINEFX) {
 	    h = MSN(fxp);
 	    l = LSN(fxp);
 	    if (l == 0xf && h != 0) {
@@ -255,7 +256,7 @@ fx_volslide:
 	 * Also don't assign xc->volslide if fxp is 0, see Guild of Sounds.xm
 	 */
 	if (fxp) {
-	    if (m->fetch & XMP_CTL_VOLPDN) {
+	    if (m->quirk & XMP_QRK_VOLPDN) {
 		if ((xc->volslide = fxp))
 	            xc->v_val = LSN(fxp) ? -LSN(fxp) : MSN(fxp);
 	    } else {
@@ -263,7 +264,7 @@ fx_volslide:
 	    }
 	}
 
-	if (m->fetch & XMP_CTL_FINEFX) {
+	if (m->quirk & XMP_QRK_FINEFX) {
 	    /* Mirko reports that a S3M with D0F effects created with ST321
 	     * should process volume slides in all frames like ST300. I
 	     * suspect ST3/IT could be handling D0F effects like this.
@@ -346,7 +347,7 @@ ex_f_porta_dn:
 		    if (--f->loop_stack[chn]) {
 			f->loop_chn = ++chn;	/* **** H:FIXME **** */
 		    } else {
-			if (m->fetch & XMP_CTL_S3MLOOP)
+			if (m->quirk & XMP_QRK_S3MLOOP)
 			    f->loop_start[chn] = f->row + 1;
 		    }
 		} else {
@@ -393,7 +394,7 @@ ex_f_vslide_dn:
 	if (fxp) {
 	    if (fxp < 0x20)	/* speedup.xm needs BPM = 20 */
 		p->tempo = fxp;
-	    else if (~m->fetch & XMP_CTL_VBLANK)
+	    else if (~m->flags & XMP_CTL_VBLANK)
 		p->tick_time = m->rrate / (p->xmp_bpm = fxp);
 	}
 	break;
@@ -402,8 +403,12 @@ ex_f_vslide_dn:
 	    p->tempo = fxp;
 	break;
     case FX_S3M_BPM:				/* Set S3M BPM */
-	if (fxp >= 0x20)	/* Panic uses 0x14 */
-	    p->tick_time = m->rrate / (p->xmp_bpm = fxp);
+	if (fxp) {
+	    if (fxp < SMIX_MINBPM)
+		fxp = SMIX_MINBPM;
+	    p->xmp_bpm = fxp;
+	    p->tick_time = m->rrate / p->xmp_bpm;
+	}
 	break;
     case FX_IT_BPM:				/* Set IT BPM */
 	if (MSN(fxp) == 0) {		/* T0x - Tempo slide down by x */
@@ -415,6 +420,8 @@ ex_f_vslide_dn:
 	    if ((int32)p->xmp_bpm > 0xff)
 		p->xmp_bpm = 0xff;
 	} else {
+	    if (fxp < SMIX_MINBPM)
+		fxp = SMIX_MINBPM;
 	    p->xmp_bpm = fxp;
 	}
 	p->tick_time = m->rrate / p->xmp_bpm;
@@ -424,7 +431,7 @@ ex_f_vslide_dn:
 	break;
     case FX_G_VOLSLIDE:				/* Global volume slide */
 	p->gvol_flag = 1;
-	if (m->fetch & XMP_CTL_FINEFX) {
+	if (m->quirk & XMP_QRK_FINEFX) {
 	    h = MSN(fxp);
 	    l = LSN(fxp);
 	    if (h == 0xf && l != 0) {
@@ -484,7 +491,7 @@ fx_xf_porta:
 	break;
     case FX_TRK_VSLIDE:				/* Track volume slide */
 fx_trk_vslide:
-	if (m->fetch & XMP_CTL_FINEFX) {
+	if (m->quirk & XMP_QRK_FINEFX) {
 	    h = MSN(fxp);
 	    l = LSN(fxp);
 	    if (h == 0xf && l != 0) {
