@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -10,7 +10,7 @@
 #include "config.h"
 #endif
 
-#include "xmpi.h"
+#include "common.h"
 #include "player.h"
 #include "driver.h"
 
