@@ -1,18 +1,16 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr.
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr.
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See docs/COPYING
  * for more information.
- *
- * $Id: mix_all.c,v 1.4 2007/10/24 20:30:17 cmatsuoka Exp $
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include "xmpi.h"
+#include "common.h"
 #include "driver.h"
 #include "mixer.h"
 #include "synth.h"
@@ -59,6 +57,29 @@
     itpt += itpt_inc; \
 } while (0)
 
+#define MIX_STEREO_AC() do { \
+    if (vi->attack) { \
+	int a = SLOW_ATTACK - vi->attack; \
+	*(tmp_bk++) += smp_in * vr * a / SLOW_ATTACK; \
+	*(tmp_bk++) += smp_in * vl * a / SLOW_ATTACK; \
+	vi->attack--; \
+    } else { \
+	*(tmp_bk++) += smp_in * vr; \
+	*(tmp_bk++) += smp_in * vl; \
+    } \
+    itpt += itpt_inc; \
+} while (0)
+
+#define MIX_MONO_AC() do { \
+    if (vi->attack) { \
+	*(tmp_bk++) += smp_in * vl * (SLOW_ATTACK - vi->attack) / SLOW_ATTACK; \
+	vi->attack--; \
+    } else { \
+	*(tmp_bk++) += smp_in * vl; \
+    } \
+    itpt += itpt_inc; \
+} while (0)
+
 #define VAR_NORM(x) \
     register int smp_in; \
     x *in_bk = vi->sptr; \
@@ -81,7 +102,7 @@
 SMIX_MIXER(smix_st8itpt)
 {
     VAR_ITPT(int8);
-    while (count--) { INTERPOLATE(); MIX_STEREO(); }
+    while (count--) { INTERPOLATE(); MIX_STEREO_AC(); }
 }
 
 
@@ -93,7 +114,7 @@ SMIX_MIXER(smix_st16itpt)
 
     vl >>= 8;
     vr >>= 8;
-    while (count--) { INTERPOLATE(); MIX_STEREO(); }
+    while (count--) { INTERPOLATE(); MIX_STEREO_AC(); }
 }
 
 
@@ -128,7 +149,7 @@ SMIX_MIXER(smix_mn8itpt)
     VAR_ITPT(int8);
 
     vl <<= 1;
-    while (count--) { INTERPOLATE(); MIX_MONO(); }
+    while (count--) { INTERPOLATE(); MIX_MONO_AC(); }
 }
 
 
@@ -139,7 +160,7 @@ SMIX_MIXER(smix_mn16itpt)
     VAR_ITPT(int16);
 
     vl >>= 7;
-    while (count--) { INTERPOLATE(); MIX_MONO(); }
+    while (count--) { INTERPOLATE(); MIX_MONO_AC(); }
 }
 
 
@@ -177,7 +198,7 @@ SMIX_MIXER(smix_st8itpt_flt)
     VAR_ITPT(int8);
     VAR_FILT;
 
-    while (count--) { INTERPOLATE(); DO_FILTER(); MIX_STEREO(); }
+    while (count--) { INTERPOLATE(); DO_FILTER(); MIX_STEREO_AC(); }
     SAVE_FILTER();
 }
 
@@ -191,7 +212,7 @@ SMIX_MIXER(smix_st16itpt_flt)
 
     vl >>= 8;
     vr >>= 8;
-    while (count--) { INTERPOLATE(); DO_FILTER(); MIX_STEREO(); }
+    while (count--) { INTERPOLATE(); DO_FILTER(); MIX_STEREO_AC(); }
     SAVE_FILTER();
 }
 
@@ -204,7 +225,7 @@ SMIX_MIXER(smix_mn8itpt_flt)
     VAR_FILT;
 
     vl <<= 1;
-    while (count--) { INTERPOLATE(); DO_FILTER(); MIX_MONO(); }
+    while (count--) { INTERPOLATE(); DO_FILTER(); MIX_MONO_AC(); }
     SAVE_FILTER();
 }
 
@@ -217,7 +238,7 @@ SMIX_MIXER(smix_mn16itpt_flt)
     VAR_FILT;
 
     vl >>= 7;
-    while (count--) { INTERPOLATE(); DO_FILTER(); MIX_MONO(); }
+    while (count--) { INTERPOLATE(); DO_FILTER(); MIX_MONO_AC(); }
     SAVE_FILTER();
 }
 
