@@ -10,10 +10,18 @@
 #include "config.h"
 #endif
 
+#include <stdlib.h>
+#include <string.h>
 #include <assert.h>
 
+#include "common.h"
+#include "driver.h"
 #include "mixer.h"
 #include "synth.h"
+#include "smix.h"
+#include "period.h"
+#include "convert.h"
+
 
 #define FLAG_ITPT	0x01
 #define FLAG_16_BITS	0x02
@@ -32,22 +40,6 @@
 #define LIM16_HI	 32767
 #define LIM16_LO	-32768
 
-#define TURN_OFF	0
-#define TURN_ON		1
-
-static char** smix_buffer = NULL;	/* array of output buffers */
-static int* smix_buf32b = NULL;		/* temp buffer for 32 bit samples */
-static int smix_numvoc;			/* default softmixer voices number */
-static int smix_numbuf;			/* number of buffer to output */
-static int smix_mode;			/* mode = 0:OFF, 1:MONO, 2:STEREO */
-static int smix_resol;			/* resolution output 1:8bit, 2:16bit */
-static int smix_ticksize;
-int **xmp_mix_buffer = &smix_buf32b;
-
-static int smix_dtright;		/* anticlick control, right channel */
-static int smix_dtleft;			/* anticlick control, left channel */
-
-static int echo_msg;
 
 void    smix_mn8norm      (struct voice_info *, int *, int, int, int, int);
 void    smix_mn8itpt      (struct voice_info *, int *, int, int, int, int);
@@ -159,19 +151,20 @@ static void out_u8ulaw(char *dest, int *src, int num, int amp, int flags)
 
 
 /* Prepare the mixer for the next tick */
-inline static void smix_resetvar(struct xmp_context *ctx)
+void smix_resetvar(struct xmp_context *ctx)
 {
     struct xmp_player_context *p = &ctx->p;
     struct xmp_mod_context *m = &p->m;
+    struct xmp_smixer_context *s = &ctx->s;
     struct xmp_options *o = &ctx->o;
 
-    smix_ticksize = m->fetch & XMP_CTL_MEDBPM ?
+    s->ticksize = m->quirk & XMP_QRK_MEDBPM ?
 	o->freq * m->rrate * 33 / p->xmp_bpm / 12500 :
     	o->freq * m->rrate / p->xmp_bpm / 100;
 
-    if (smix_buf32b) {
-	smix_dtright = smix_dtleft = TURN_OFF;
-	memset(smix_buf32b, 0, smix_ticksize * smix_mode * sizeof (int));
+    if (s->buf32b) {
+	s->dtright = s->dtleft = 0;
+	memset(s->buf32b, 0, s->ticksize * s->mode * sizeof (int));
     }
 }
 
@@ -180,13 +173,14 @@ inline static void smix_resetvar(struct xmp_context *ctx)
 static void smix_rampdown(struct xmp_context *ctx, int voc, int32 *buf, int cnt)
 {
     struct xmp_driver_context *d = &ctx->d;
+    struct xmp_smixer_context *s = &ctx->s;
     int smp_l, smp_r;
     int dec_l, dec_r;
 
     if (voc < 0) {
 	/* initialize */
-	smp_r = smix_dtright;
-	smp_l = smix_dtleft;
+	smp_r = s->dtright;
+	smp_l = s->dtleft;
     } else {
 	smp_r = d->voice_array[voc].sright;
 	smp_l = d->voice_array[voc].sleft;
@@ -197,8 +191,8 @@ static void smix_rampdown(struct xmp_context *ctx, int voc, int32 *buf, int cnt)
 	return;
 
     if (!buf) {
-	buf = smix_buf32b;
-	cnt = SLOW_RELEASE; //smix_ticksize;
+	buf = s->buf32b;
+	cnt = SLOW_RELEASE; //s->ticksize;
     }
     if (!cnt)
 	return;
@@ -225,9 +219,10 @@ static void smix_anticlick(struct xmp_context *ctx, int voc, int vol, int pan, i
 {
     int oldvol, newvol, pan0;
     struct xmp_driver_context *d = &ctx->d;
+    struct xmp_smixer_context *s = &ctx->s;
     struct voice_info *vi = &d->voice_array[voc];
 
-    if (extern_drv)
+    if (d->ext)
 	return;			/* Anticlick is useful for softmixer only */
 
     /* From: Mirko Buffoni <mirbuf@gmail.com>
@@ -253,9 +248,9 @@ static void smix_anticlick(struct xmp_context *ctx, int voc, int vol, int pan, i
     }
 
     if (!buf) {
-	smix_dtright += vi->sright;
-	smix_dtleft += vi->sleft;
-	vi->sright = vi->sleft = TURN_OFF;
+	s->dtright += vi->sright;
+	s->dtleft += vi->sleft;
+	vi->sright = vi->sleft = 0;
     } else {
 	smix_rampdown(ctx, voc, buf, cnt);
     }
@@ -265,9 +260,10 @@ static void smix_anticlick(struct xmp_context *ctx, int voc, int vol, int pan, i
 /* Fill the output buffer calling one of the handlers. The buffer contains
  * sound for one tick (a PAL frame or 1/50s for standard vblank-timed mods)
  */
-static int softmixer(struct xmp_context *ctx)
+int xmp_smix_softmixer(struct xmp_context *ctx)
 {
     struct xmp_driver_context *d = &ctx->d;
+    struct xmp_smixer_context *s = &ctx->s;
     struct voice_info *vi;
     struct patch_info *pi;
     int smp_cnt, tic_cnt, lps, lpe;
@@ -276,27 +272,27 @@ static int softmixer(struct xmp_context *ctx)
     int synth = 1;
     int *buf_pos;
 
-    if (!extern_drv)
+    if (!d->ext)
 	smix_rampdown(ctx, -1, NULL, 0);	/* Anti-click */
 
-    for (voc = numvoc; voc--; ) {
+    for (voc = d->maxvoc; voc--; ) {
 	vi = &d->voice_array[voc];
 
 	if (vi->chn < 0)
 	    continue;
 
 	if (vi->period < 1) {
-	    drv_resetvoice(ctx, voc, 1);
+	    xmp_drv_resetvoice(ctx, voc, 1);
 	    continue;
 	}
 
-	buf_pos = smix_buf32b;
+	buf_pos = s->buf32b;
 	vol_r = vi->vol * (0x80 - vi->pan);
 	vol_l = vi->vol * (0x80 + vi->pan);
 
 	if (vi->fidx & FLAG_SYNTH) {
 	    if (synth) {
-		smix_synth(vi, buf_pos, smix_ticksize, vol_l, vol_r,
+		smix_synth(vi, buf_pos, s->ticksize, vol_l, vol_r,
 						vi->fidx & FLAG_STEREO);
 	        synth = 0;
 	    }
@@ -330,7 +326,7 @@ static int softmixer(struct xmp_context *ctx)
 		lpe >>= 1;
 	}
 
-	for (tic_cnt = smix_ticksize; tic_cnt; ) {
+	for (tic_cnt = s->ticksize; tic_cnt; ) {
 	    /* How many samples we can write before the loop break or
 	     * sample end... */
 	    smp_cnt = 1 + (((int64)(vi->end - vi->pos) << SMIX_SHIFT)
@@ -354,7 +350,7 @@ static int softmixer(struct xmp_context *ctx)
 
 	    if (vi->vol) {
 		int idx;
-		int mix_size = smix_mode * smp_cnt;
+		int mix_size = s->mode * smp_cnt;
 		int mixer = vi->fidx & FIDX_FLAGMASK;
 
 		/* Something for Hipolito's anticlick routine */
@@ -373,7 +369,7 @@ static int softmixer(struct xmp_context *ctx)
 
 		/* Call the output handler */
 		mix_fn[mixer](vi, buf_pos, smp_cnt, vol_l, vol_r, itp_inc);
-		buf_pos += smix_mode * smp_cnt;
+		buf_pos += s->mode * smp_cnt;
 
 		/* More stuff for Hipolito's anticlick routine */
 		idx = 0;
@@ -396,7 +392,7 @@ static int softmixer(struct xmp_context *ctx)
 	    /* First sample loop run */
             if (vi->fidx == 0 || lps >= lpe) {
 		smix_anticlick(ctx, voc, 0, 0, buf_pos, tic_cnt);
-		drv_resetvoice(ctx, voc, 0);
+		xmp_drv_resetvoice(ctx, voc, 0);
 		tic_cnt = 0;
 		continue;
 	    }
@@ -418,11 +414,11 @@ static int softmixer(struct xmp_context *ctx)
 	}
     }
 
-    return smix_ticksize * smix_mode * smix_resol;
+    return s->ticksize * s->mode * s->resol;
 }
 
 
-static void smix_voicepos(struct xmp_context *ctx, int voc, int pos, int itp)
+void smix_voicepos(struct xmp_context *ctx, int voc, int pos, int itp)
 {
     struct xmp_driver_context *d = &ctx->d;
     struct voice_info *vi = &d->voice_array[voc];
@@ -454,7 +450,7 @@ static void smix_voicepos(struct xmp_context *ctx, int voc, int pos, int itp)
 }
 
 
-static void smix_setpatch(struct xmp_context *ctx, int voc, int smp)
+void smix_setpatch(struct xmp_context *ctx, int voc, int smp)
 {
     struct xmp_player_context *p = &ctx->p;
     struct xmp_driver_context *d = &ctx->d;
@@ -470,7 +466,7 @@ static void smix_setpatch(struct xmp_context *ctx, int voc, int smp)
     if (pi->len == XMP_PATCH_FM) {
 	vi->fidx = FLAG_SYNTH;
 	if (o->outfmt & XMP_FMT_MONO)
-	    vi->pan = TURN_OFF;
+	    vi->pan = 0;
 	else {
 	    vi->pan = pi->panning;
 	    vi->fidx |= FLAG_STEREO;
@@ -483,11 +479,11 @@ static void smix_setpatch(struct xmp_context *ctx, int voc, int smp)
     
     xmp_smix_setvol(ctx, voc, 0);
 
-    vi->sptr = extern_drv ? NULL : pi->data;
-    vi->fidx = m->fetch & XMP_CTL_ITPT ? FLAG_ITPT | FLAG_ACTIVE : FLAG_ACTIVE;
+    vi->sptr = d->ext ? NULL : pi->data;
+    vi->fidx = m->flags & XMP_CTL_ITPT ? FLAG_ITPT | FLAG_ACTIVE : FLAG_ACTIVE;
 
     if (o->outfmt & XMP_FMT_MONO) {
-	vi->pan = TURN_OFF;
+	vi->pan = 0;
     } else {
 	vi->pan = pi->panning;
 	vi->fidx |= FLAG_STEREO;
@@ -496,7 +492,7 @@ static void smix_setpatch(struct xmp_context *ctx, int voc, int smp)
     if (pi->mode & WAVE_16_BITS)
 	vi->fidx |= FLAG_16_BITS;
 
-    if (m->fetch & XMP_CTL_FILTER)
+    if (m->flags & XMP_CTL_FILTER)
 	vi->fidx |= FLAG_FILTER;
 
     if (pi->mode & WAVE_LOOPING)
@@ -514,7 +510,7 @@ static void smix_setpatch(struct xmp_context *ctx, int voc, int smp)
 }
 
 
-static void smix_setnote(struct xmp_context *ctx, int voc, int note)
+void smix_setnote(struct xmp_context *ctx, int voc, int note)
 {
     struct xmp_driver_context *d = &ctx->d;
     struct voice_info *vi = &d->voice_array[voc];
@@ -525,7 +521,7 @@ static void smix_setnote(struct xmp_context *ctx, int voc, int note)
 }
 
 
-static inline void smix_setbend(struct xmp_context *ctx, int voc, int bend)
+void smix_setbend(struct xmp_context *ctx, int voc, int bend)
 {
     struct xmp_driver_context *d = &ctx->d;
     struct voice_info *vi = &d->voice_array[voc];
@@ -584,21 +580,26 @@ void xmp_smix_setpan(struct xmp_context *ctx, int voc, int pan)
 }
 
 
-void xmp_smix_echoback(int msg)
+void xmp_smix_echoback(struct xmp_context *ctx, int msg)
 {
-    xmp_event_callback(echo_msg = msg);
+    struct xmp_player_context *p = &ctx->p;
+    struct xmp_smixer_context *s = &ctx->s;
+
+    p->event_callback(s->echo_msg = msg);
 }
 
 
-int xmp_smix_getmsg()
+int xmp_smix_getmsg(struct xmp_context *ctx)
 {
-    return echo_msg;
+    struct xmp_smixer_context *s = &ctx->s;
+    return s->echo_msg;
 }
 
 
-int xmp_smix_numvoices(int num)
+int xmp_smix_numvoices(struct xmp_context *ctx, int num)
 {
-    return num > smix_numvoc ? smix_numvoc : num;
+    struct xmp_smixer_context *s = &ctx->s;
+    return num > s->numvoc ? s->numvoc : num;
 }
 
 /* WARNING! Output samples must have the same byte order of the host machine!
@@ -625,42 +626,46 @@ int xmp_smix_writepatch(struct xmp_context *ctx, struct patch_info *patch)
 int xmp_smix_on(struct xmp_context *ctx)
 {
     struct xmp_driver_context *d = &ctx->d;
+    struct xmp_smixer_context *s = &ctx->s;
     int cnt;
 
-    if (smix_numbuf)
+    if (s->numbuf)
 	return 0;
 
     if (d->numbuf < 1)
 	d->numbuf = 1;
-    cnt = smix_numbuf = d->numbuf;
+    cnt = s->numbuf = d->numbuf;
 
-    smix_buffer = calloc(sizeof (void *), cnt);
-    smix_buf32b = calloc(sizeof (int), OUT_MAXLEN);
-    if (!(smix_buffer && smix_buf32b))
+    s->buffer = calloc(sizeof (void *), cnt);
+    s->buf32b = calloc(sizeof (int), OUT_MAXLEN);
+    if (!(s->buffer && s->buf32b))
 	return XMP_ERR_ALLOC;
 
     while (cnt--) {
-	if (!(smix_buffer[cnt] = calloc(SMIX_RESMAX, OUT_MAXLEN)))
+	if (!(s->buffer[cnt] = calloc(SMIX_RESMAX, OUT_MAXLEN)))
 	    return XMP_ERR_ALLOC;
     }
 
-    smix_numvoc = SMIX_NUMVOC;
-    extern_drv = TURN_OFF;
+    s->numvoc = SMIX_NUMVOC;
+    d->ext = 0;
 
     return 0;
 }
 
 
-void xmp_smix_off()
+void xmp_smix_off(struct xmp_context *ctx)
 {
-    while (smix_numbuf)
-	free(smix_buffer[--smix_numbuf]);
+    struct xmp_driver_context *d = &ctx->d;
+    struct xmp_smixer_context *s = &ctx->s;
 
-    free(smix_buf32b);
-    free(smix_buffer);
-    smix_buf32b = NULL;
-    smix_buffer = NULL;
-    extern_drv = TURN_ON;
+    while (s->numbuf)
+	free(s->buffer[--s->numbuf]);
+
+    free(s->buf32b);
+    free(s->buffer);
+    s->buf32b = NULL;
+    s->buffer = NULL;
+    d->ext = 1;
 }
 
 
@@ -668,6 +673,7 @@ void *xmp_smix_buffer(struct xmp_context *ctx)
 {
     static int outbuf;
     int fmt, size;
+    struct xmp_smixer_context *s = &ctx->s;
     struct xmp_options *o = &ctx->o;
 
     if (!o->resol)
@@ -681,16 +687,16 @@ void *xmp_smix_buffer(struct xmp_context *ctx)
      * multi-buffered sound output (e.g. OSS fragments) but can be useful for
      * DMA transfers in DOS.
      */
-    if (++outbuf >= smix_numbuf)
+    if (++outbuf >= s->numbuf)
 	outbuf = 0;
 
-    size = smix_mode * smix_ticksize;
+    size = s->mode * s->ticksize;
     assert(size <= OUT_MAXLEN);
 
-    out_fn[fmt](smix_buffer[outbuf], smix_buf32b, size, o->amplify, o->outfmt);
+    out_fn[fmt](s->buffer[outbuf], s->buf32b, size, o->amplify, o->outfmt);
 
     smix_resetvar(ctx);
 
-    return smix_buffer[outbuf]; 
+    return s->buffer[outbuf]; 
 }
 
