@@ -37,13 +37,13 @@ static int period_amiga[] = {
 
 
 /* Get period from note */
-inline int note_to_period(int n, int f, int type)
+inline double note_to_period(int n, int f, int type)
 {
     double d = (double)n + (double)f / 128;
 
     return type ?
 	(120.0 - d) * 16 :			/* Linear */
-        (int)(6847.0 / pow(2, d / 12));		/* Amiga */
+        6847.0 / pow(2, d / 12);		/* Amiga */
 }
 
 
@@ -71,7 +71,7 @@ int period_to_note(int p)
 
 
 /* Get pitchbend from base note and period */
-int period_to_bend(int p, int n, int limit, int gliss, int type)
+int period_to_bend(double p, int n, int limit, int gliss, int type)
 {
     int b;
     double d;
