@@ -37,6 +37,15 @@
 #include "synth.h"
 
 
+/* Values for multi-retrig */
+static struct retrig_t rval[] = {
+    {   0,  1,  1 }, {  -1,  1,  1 }, {  -2,  1,  1 }, {  -4,  1,  1 },
+    {  -8,  1,  1 }, { -16,  1,  1 }, {   0,  2,  3 }, {   0,  1,  2 },
+    {   0,  1,  1 }, {   1,  1,  1 }, {   2,  1,  1 }, {   4,  1,  1 },
+    {   8,  1,  1 }, {  16,  1,  1 }, {   0,  3,  2 }, {   0,  2,  1 },
+    {   0,  0,  1 }	/* Note cut */
+};
+
 /* Vibrato/tremolo waveform tables */
 static int waveform[4][64] = {
    {   0,  24,  49,  74,  97, 120, 141, 161, 180, 197, 212, 224,
@@ -77,7 +86,7 @@ static int waveform[4][64] = {
 static int fetch_channel (struct xmp_context *, struct xxm_event *, int, int);
 static void play_channel (struct xmp_context *, int, int);
 
-#include "effects.c"
+/* #include "effects.c" */
 
 
 static void dummy(unsigned long ul)
@@ -131,7 +140,7 @@ static int do_envelope(struct xmp_context *ctx, struct xxm_envinfo *ei, uint16 *
     else
 	loop = ei->flg & XXM_ENV_LOOP;
 
-    if (m->fetch & XMP_CTL_ITENV) {
+    if (HAS_QUIRK(XMP_QRK_ITENV)) {
 	if (!rl && (ei->flg & XXM_ENV_SUS)) {
 	    if (*x >= env[ei->sue << 1])
 		*x = env[ei->sus << 1];
@@ -155,7 +164,7 @@ static int do_envelope(struct xmp_context *ctx, struct xxm_envinfo *ei, uint16 *
 	if (!env[rl + 1])
 	    xmp_drv_resetchannel(ctx, chn);
 	else
-	    return m->fetch & XMP_CTL_ENVFADE;
+	    return HAS_QUIRK(XMP_QRK_ENVFADE);
     }
 
     return 0;
@@ -267,7 +276,7 @@ static int fetch_channel(struct xmp_context *ctx, struct xxm_event *e, int chn,
     }
 
     /* Emulate Impulse Tracker "always read instrument" bug */
-    if (e->note && !e->ins && xc->delayed_ins && m->fetch & XMP_CTL_SAVEINS) {
+    if (e->note && !e->ins && xc->delayed_ins && HAS_QUIRK(XMP_QRK_SAVEINS)) {
 	e->ins = xc->delayed_ins;
 	xc->delayed_ins = 0;
     }
@@ -286,13 +295,13 @@ static int fetch_channel(struct xmp_context *ctx, struct xxm_event *e, int chn,
 	xc->fadeout = 0x8000;	/* for painlace.mod pat 0 ch 3 echo */
 	xc->per_flags = 0;
 
-	if (m->fetch & XMP_CTL_OINSMOD) {
+	if (HAS_QUIRK(XMP_QRK_OINSMOD)) {
 	    if (TEST(IS_READY)) {
 		xins = xc->insdef;
 		RESET(IS_READY);
 	    }
 	} else if ((uint32)ins < m->xxh->ins && m->xxih[ins].nsm) {	/* valid ins */
-	    if (!key && m->fetch & XMP_CTL_INSPRI) {
+	    if (!key && HAS_QUIRK(XMP_QRK_INSPRI)) {
 		if (xins == ins)
 		    flg = NEW_INS | RESET_VOL;
 		else 
@@ -300,10 +309,10 @@ static int fetch_channel(struct xmp_context *ctx, struct xxm_event *e, int chn,
 	    }
 	    xins = ins;
 	} else {					/* invalid ins */
-	    if (~m->fetch & XMP_CTL_NCWINS)
+	    if (!HAS_QUIRK(XMP_QRK_NCWINS))
 		xmp_drv_resetchannel(ctx, chn);
 
-	    if (m->fetch & XMP_CTL_IGNWINS) {
+	    if (HAS_QUIRK(XMP_QRK_IGNWINS)) {
 		ins = -1;
 		flg = 0;
 	    }
@@ -326,9 +335,10 @@ static int fetch_channel(struct xmp_context *ctx, struct xxm_event *e, int chn,
 	} else if (key == XMP_KEY_OFF) {
 	    SET(RELEASE);
 	    flg &= ~(RESET_VOL | RESET_ENV);
-	} else if (e->fxt == FX_TONEPORTA || e->f2t == FX_TONEPORTA) {
-	    /* This test should fix portamento behaviour in 7spirits.s3m */
-	    if (m->fetch & XMP_CTL_RTGINS && e->ins && xc->ins != ins) {
+	} else if (e->fxt == FX_TONEPORTA || e->f2t == FX_TONEPORTA
+		|| e->fxt == FX_TONE_VSLIDE || e->f2t == FX_TONE_VSLIDE) {
+	    /* Fix portamento in 7spirits.s3m and mod.Biomechanoid */
+	    if (HAS_QUIRK(XMP_QRK_RTGINS) && e->ins && xc->ins != ins) {
 		flg |= NEW_INS;
 		xins = ins;
 	    } else {
@@ -379,7 +389,7 @@ static int fetch_channel(struct xmp_context *ctx, struct xxm_event *e, int chn,
 		flg &= ~(RESET_VOL | RESET_ENV | NEW_INS | NEW_NOTE);
 	    }
 	} else {
-	    if (!(m->fetch & XMP_CTL_CUTNWI))
+	    if (!HAS_QUIRK(XMP_QRK_CUTNWI))
 		xmp_drv_resetchannel(ctx, chn);
 	}
     }
@@ -419,14 +429,14 @@ static int fetch_channel(struct xmp_context *ctx, struct xxm_event *e, int chn,
 	SET(NEW_VOL);
     }
 
-    if (TEST(NEW_INS) || (m->fetch & XMP_CTL_OFSRST))
+    if (TEST(NEW_INS) || HAS_QUIRK(XMP_QRK_OFSRST))
 	xc->offset_val = 0;
 
     /* Secondary effect is processed _first_ and can be overriden
      * by the primary effect.
      */
-    process_fx(ctx, chn, e->note, e->f2t, e->f2p, xc);
-    process_fx(ctx, chn, e->note, e->fxt, e->fxp, xc);
+    process_fx(ctx, chn, e->note, e->f2t, e->f2p, xc, 1);
+    process_fx(ctx, chn, e->note, e->fxt, e->fxp, xc, 0);
 
     if (!TEST(IS_VALID)) {
 	xc->volume = 0;
@@ -438,7 +448,7 @@ static int fetch_channel(struct xmp_context *ctx, struct xxm_event *e, int chn,
 
 	if (cont_sample == 0) {
 	    xmp_drv_voicepos(ctx, chn, xc->offset_val);
-	    if (TEST(OFFSET) && (m->fetch & XMP_CTL_FX9BUG))
+	    if (TEST(OFFSET) && HAS_QUIRK(XMP_QRK_FX9BUG))
 		xc->offset_val <<= 1;
 	}
 	RESET(OFFSET);
@@ -477,7 +487,7 @@ static int fetch_channel(struct xmp_context *ctx, struct xxm_event *e, int chn,
 	SET(ECHOBACK | NEW_VOL);
     }
 
-    if ((m->fetch & XMP_CTL_ST3GVOL) && TEST(NEW_VOL) && m->volbase)
+    if (HAS_QUIRK(XMP_QRK_ST3GVOL) && TEST(NEW_VOL) && m->volbase)
 	xc->volume = xc->volume * m->volume / m->volbase;
 
     return 0;
@@ -515,7 +525,7 @@ static void play_channel(struct xmp_context *ctx, int chn, int t)
 	xc->delay = xc->retrig = xc->a_idx = 0;
 	xc->a_size = 1;
 	xc->a_val[0] = 0;
-	xc->flags &= (0xff000000 | IS_VALID);
+	xc->flags &= (0xff000000 | IS_VALID);	/* keep persistent flags */
     }
 
     if (!TEST(IS_VALID))
@@ -539,7 +549,7 @@ static void play_channel(struct xmp_context *ctx, int chn, int t)
 	     * Only reset the channel in virtual channel mode so we
 	     * can release it.
 	     */
-	     if (m->fetch & XMP_CTL_VIRTUAL) {
+	     if (m->flags & XMP_CTL_VIRTUAL) {
 		 xmp_drv_resetchannel(ctx, chn);
 		 return;
 	     } else {
@@ -597,7 +607,7 @@ static void play_channel(struct xmp_context *ctx, int chn, int t)
     finalvol = (finalvol * xc->fadeout) >> 5;	/* 16 bit output */
 
     finalvol = (uint32) (vol_envelope *
-	(m->fetch & XMP_CTL_ST3GVOL ? 0x40 : m->volume) *
+	(HAS_QUIRK(XMP_QRK_ST3GVOL) ? 0x40 : m->volume) *
 	xc->mastervol / 0x40 * ((int)finalvol * 0x40 / p->gvol_base)) >> 18;
 
     /* Volume translation table (for PTM) */
@@ -636,7 +646,7 @@ static void play_channel(struct xmp_context *ctx, int chn, int t)
      *       it may be a bug.
      */
 
-    finalpan = m->fetch & XMP_CTL_DYNPAN ?  xc->pan + (pan_envelope - 32) *
+    finalpan = m->flags & XMP_CTL_DYNPAN ?  xc->pan + (pan_envelope - 32) *
 			(128 - abs (xc->pan - 128)) / 32 : 0x80;
     finalpan = xc->masterpan + (finalpan - 128) *
 			(128 - abs (xc->masterpan - 128)) / 128;
@@ -662,17 +672,6 @@ static void play_channel(struct xmp_context *ctx, int chn, int t)
 	}
     }
 
-#if 0
-    /* Do delay */
-    if (xc->delay) {
-	if (--xc->delay) {
-	    finalvol = 0;
-	} else {
-	    xmp_drv_retrig(ctx, chn);
-	}
-    }
-#endif
-
     /* Do tremor */
     if (xc->tcnt_up || xc->tcnt_dn) {
 	if (xc->tcnt_up > 0) {
@@ -694,7 +693,7 @@ static void play_channel(struct xmp_context *ctx, int chn, int t)
     /* Volume slides happen in all frames but the first, except when the
      * "volume slide on all frames" flag is set.
      */
-    if (t % p->tempo || m->fetch & XMP_CTL_VSALL) {
+    if (t % p->tempo || HAS_QUIRK(XMP_QRK_VSALL)) {
 	if (!chn && p->gvol_flag) {
 	    m->volume += p->gvol_slide;
 	    if (m->volume < 0)
@@ -726,7 +725,7 @@ static void play_channel(struct xmp_context *ctx, int chn, int t)
     /* "Fine" sliding effects are processed in the first frame of each row,
      * and standard slides in the rest of the frames.
      */
-    if (t % p->tempo || m->fetch & XMP_CTL_PBALL) {
+    if (t % p->tempo || HAS_QUIRK(XMP_QRK_PBALL)) {
 	/* Do pan and pitch sliding */
 	if (TEST(PAN_SLIDE)) {
 	    xc->pan += xc->p_val;
@@ -812,10 +811,10 @@ static void play_channel(struct xmp_context *ctx, int chn, int t)
     finalpan = o->outfmt & XMP_FMT_MONO ?
 	0 : (finalpan - 0x80) * o->mix / 100;
     xmp_drv_setbend(ctx, chn, xc->pitchbend + xc->a_val[xc->a_idx] + med_arp);
-    xmp_drv_setpan(ctx, chn, m->fetch & XMP_CTL_REVERSE ? -finalpan : finalpan);
+    xmp_drv_setpan(ctx, chn, m->flags & XMP_CTL_REVERSE ? -finalpan : finalpan);
     xmp_drv_setvol(ctx, chn, finalvol);
 
-    if (cutoff < 0xff && (m->fetch & XMP_CTL_FILTER)) {
+    if (cutoff < 0xff && (m->flags & XMP_CTL_FILTER)) {
 	filter_setup(ctx, xc, cutoff);
 	xmp_drv_seteffect(ctx, chn, XMP_FX_FILTER_B0, xc->flt_B0);
 	xmp_drv_seteffect(ctx, chn, XMP_FX_FILTER_B1, xc->flt_B1);
@@ -833,230 +832,22 @@ static void play_channel(struct xmp_context *ctx, int chn, int t)
 
 int _xmp_player_start(struct xmp_context *ctx)
 {
-    int num_rows, frame, e, ord;	/* rows, frame, end point, order */
-    int ret;
-    double playing_time;
-    struct xmp_player_context *p = &ctx->p;
-    struct xmp_driver_context *d = &ctx->d;
-    struct xmp_mod_context *m = &p->m;
-    struct xmp_options *o = &ctx->o;
-    struct flow_control *f = &p->flow;
-
-    if (m->xxh->len == 0 || m->xxh->chn == 0) {
-	xmp_drv_writepatch(ctx, NULL);
-	return 0;
-    }
-
-    if (xmp_event_callback == NULL)
-	xmp_event_callback = dummy;
-
-    p->gvol_slide = 0;
-    p->gvol_base = m->volbase;
-    ord = o->start;
-
-    /* Skip invalid patterns at start (the seventh laboratory.it) */
-    while (ord < m->xxh->len && m->xxo[ord] >= m->xxh->pat)
-	ord++;
-
-    m->volume = m->xxo_info[ord].gvl;
-    p->tick_time = m->rrate / (p->xmp_bpm = m->xxo_info[ord].bpm);
-    f->jumpline = m->xxo_fstrow[ord];
-    p->tempo = m->xxo_info[ord].tempo;
-    playing_time = 0;
-
-    if ((ret = xmp_drv_on(ctx, m->xxh->chn)) != 0)
-	return ret;
-
-    f->jump = -1;
-
-    p->fetch_ctl = calloc(m->xxh->chn, sizeof (int));
-    f->loop_stack = calloc(d->numchn, sizeof (int));
-    f->loop_start = calloc(d->numchn, sizeof (int));
-    p->xc_data = calloc(d->numchn, sizeof (struct xmp_channel));
-    if (!(p->fetch_ctl && f->loop_stack && f->loop_start && p->xc_data))
-	return XMP_ERR_ALLOC;
-
-    reset_channel(ctx);
-
-    xmp_drv_starttimer(ctx);
-
-    for (e = p->xmp_scan_num; ; ord++) {
-next_order:
-	if (ord >= m->xxh->len) {
-	    ord = ((uint32)m->xxh->rst > m->xxh->len ||
-		(uint32)m->xxo[m->xxh->rst] >= m->xxh->pat) ? 0 : m->xxh->rst;
-	    m->volume = m->xxo_info[ord].gvl;
-	}
-
-	/* Skip invalid patterns */
-	if (m->xxo[ord] >= m->xxh->pat) {
-#if 0
-	   if (m->xxo[ord] == 0xff)	    /* S3M uses 0xff as end mark */
-		ord = m->xxh->len;
-#endif
-	    continue;
-	}
-
-	num_rows = m->xxp[m->xxo[ord]]->rows;
-	if (f->jumpline >= num_rows)
-	    f->jumpline = 0;
-	f->row = f->jumpline;
-	f->jumpline = 0;
-
-	p->pos = ord;
-
-	/* Reset persistent effects at each new pattern */
-	if (m->fetch & XMP_CTL_PERPAT) {
-	    int chn;
-	    for (chn = 0; chn < p->m.xxh->chn; chn++)
-		p->xc_data[chn].per_flags = 0;
-	}
-
-	/* Row processing */
-	for (; f->row < num_rows; f->row++) {
-
-	    /* Check module end */
-	    if ((~m->fetch & XMP_CTL_LOOP) && ord == p->xmp_scan_ord &&
-		f->row == p->xmp_scan_row) {
-		if (!e--)
-		    goto end_module;
-	    }
-
-	    if (p->pause) {
-		xmp_drv_stoptimer(ctx);
-		while (p->pause) {
-		    usleep(125000);
-		    xmp_event_callback(0);
-		}
-		xmp_drv_starttimer(ctx);
-	    }
-
-	    /* Frame processing */
-	    for (frame = 0; frame < (p->tempo * (1 + f->delay)); frame++) {
-
-		/* Check reposition */
-		if (ord != p->pos) {
-		    if (p->pos == -1)
-			p->pos++;		/* restart module */
-
-		    if (p->pos == -2) {		/* set by xmp_module_stop */
-			xmp_drv_bufwipe(ctx);
-			goto end_module;	/* that's all folks */
-		    }
-
-		    if (p->pos == 0)
-			e = p->xmp_scan_num;
-
-		    p->tempo = m->xxo_info[ord = p->pos].tempo;
-		    p->tick_time = m->rrate / (p->xmp_bpm = m->xxo_info[ord].bpm);
-		    m->volume = m->xxo_info[ord].gvl;
-		    f->jump = ord;
-		    f->jumpline = m->xxo_fstrow[ord--];
-		    f->row = -1;
-		    xmp_drv_bufwipe(ctx);
-		    xmp_drv_sync(ctx, 0);
-		    xmp_drv_reset(ctx);
-		    reset_channel(ctx);
-		    break;
-		}
-
-		if (frame == 0) {		/* first frame of row */
-		    p->gvol_flag = 0;
-
-		    fetch_row(ctx, m->xxo[ord], f->row);
-
-		    xmp_drv_echoback(ctx, (p->tempo << 12) |
-					(p->xmp_bpm << 4) | XMP_ECHO_BPM);
-		    xmp_drv_echoback(ctx, (m->volume << 4) | XMP_ECHO_GVL);
-		    xmp_drv_echoback(ctx, (m->xxo[ord] << 12) |
-					(ord << 4) | XMP_ECHO_ORD);
-		    xmp_drv_echoback(ctx, (d->numvoc << 4) | XMP_ECHO_NCH);
-		    xmp_drv_echoback(ctx, ((m->xxp[m->xxo[ord]]->rows - 1)
-				<< 12) | (f->row << 4) | XMP_ECHO_ROW);
-		}
-
-		xmp_drv_echoback(ctx, (frame << 4) | XMP_ECHO_FRM);
-
-		play_frame(ctx, frame);
-
-		if (o->time && (o->time < playing_time))
-		    goto end_module;
-
-		if (m->fetch & XMP_CTL_MEDBPM) {
-		    xmp_drv_sync(ctx, p->tick_time * 33 / 125);
-		    playing_time += m->rrate * 33 / (100 * p->xmp_bpm * 125);
-		} else {
-		    xmp_drv_sync(ctx, p->tick_time);
-		    playing_time += m->rrate / (100 * p->xmp_bpm);
-		}
-
-		xmp_drv_bufdump(ctx);
-	    }
-
-	    f->delay = 0;
-
-	    if (f->row == -1)
-		break;
-
-	    if (f->pbreak) {
-		f->pbreak = 0;
-		break;
-	    }
-
-	    if (f->loop_chn) {
-		f->row = f->loop_start[--f->loop_chn] - 1;
-		f->loop_chn = 0;
-	    }
-	}
-
-	if (f->jump != -1) {
-	    ord = f->jump;
-	    f->jump = -1;
-	    goto next_order;
-	}
-    }
-
-end_module:
-    xmp_drv_echoback(ctx, XMP_ECHO_END);
-    while (xmp_drv_getmsg(ctx) != XMP_ECHO_END)
-	xmp_drv_bufdump(ctx);
-    xmp_drv_stoptimer(ctx);
-
-    free(p->xc_data);
-    free(f->loop_start);
-    free(f->loop_stack);
-    free(p->fetch_ctl);
-
-    xmp_drv_off(ctx);
-
-    return 0;
-}
-
-
-#ifdef TEST_OPEN_LOOP
-/*
- * EXPERIMENTAL open loop
- */
-
-int xmp_player_start(struct xmp_context *ctx)
-{
-    //int r, t, e;		/* rows, tick, end point, order */
-	int ret;
 	struct xmp_player_context *p = &ctx->p;
 	struct xmp_driver_context *d = &ctx->d;
 	struct xmp_mod_context *m = &p->m;
 	struct xmp_options *o = &ctx->o;
 	struct flow_control *f = &p->flow;
+	int ret;
 
 	if (m->xxh->len == 0 || m->xxh->chn == 0)
 		return 0;
 
-	if (xmp_event_callback == NULL)
-		xmp_event_callback = dummy;
+	if (p->event_callback == NULL)
+		p->event_callback = dummy;
 
 	p->gvol_slide = 0;
 	p->gvol_base = m->volbase;
-	f->ord = o->start;
+	p->pos = f->ord = o->start;
 	f->frame = 0;
 	f->row = 0;
 	f->num_rows = m->xxp[m->xxo[f->ord]]->rows;
@@ -1092,7 +883,7 @@ int xmp_player_start(struct xmp_context *ctx)
 }
 
 
-int xmp_player_loop(struct xmp_context *ctx)
+int _xmp_player_frame(struct xmp_context *ctx)
 {
 	struct xmp_player_context *p = &ctx->p;
 	struct xmp_driver_context *d = &ctx->d;
@@ -1100,9 +891,48 @@ int xmp_player_loop(struct xmp_context *ctx)
 	struct xmp_options *o = &ctx->o;
 	struct flow_control *f = &p->flow;
 
+	if (p->pause) {
+		p->event_callback(0);
+		return 0;
+	}
+
+	/* check reposition */
+	if (f->ord != p->pos) {
+		if (p->pos == -1)
+			p->pos++;		/* restart module */
+
+		if (p->pos == -2) {		/* set by xmp_module_stop */
+			xmp_drv_bufwipe(ctx);
+			return -1;		/* that's all folks */
+		}
+
+		if (p->pos == 0)
+			f->end_point = p->xmp_scan_num;
+
+		f->ord = p->pos;
+		if (m->xxo_info[f->ord].tempo)
+			p->tempo = m->xxo_info[f->ord].tempo;
+		p->xmp_bpm = m->xxo_info[f->ord].bpm;
+		p->tick_time = m->rrate / p->xmp_bpm;
+		m->volume = m->xxo_info[f->ord].gvl;
+		f->jump = f->ord;
+		f->time = (double)m->xxo_info[f->ord].time / 1000;
+		f->jumpline = m->xxo_fstrow[f->ord];
+		f->row = -1;
+		f->pbreak = 1;
+		f->ord--;
+		xmp_drv_bufwipe(ctx);
+		xmp_drv_sync(ctx, 0);
+		xmp_drv_reset(ctx);
+		reset_channel(ctx);
+		goto next_row;
+	}
+
+	/* check new row */
+
 	if (f->frame == 0) {			/* first frame in row */
 		/* check end of module */
-	    	if ((~m->fetch & XMP_CTL_LOOP) && f->ord == p->xmp_scan_ord &&
+	    	if ((~m->flags & XMP_CTL_LOOP) && f->ord == p->xmp_scan_ord &&
 					f->row == p->xmp_scan_row) {
 			if (!f->end_point--)
 				return -1;
@@ -1116,37 +946,13 @@ int xmp_player_loop(struct xmp_context *ctx)
 		xmp_drv_echoback(ctx, (m->volume << 4) | XMP_ECHO_GVL);
 		xmp_drv_echoback(ctx, (m->xxo[f->ord] << 12) | (f->ord << 4) |
 							XMP_ECHO_ORD);
-		xmp_drv_echoback(ctx, (d->numvoc << 4) | XMP_ECHO_NCH);
+		xmp_drv_echoback(ctx, (d->curvoc << 4) | XMP_ECHO_NCH);
 		xmp_drv_echoback(ctx, ((m->xxp[m->xxo[f->ord]]->rows - 1)
 				<< 12) | (f->row << 4) | XMP_ECHO_ROW);
+		xmp_drv_echoback(ctx, ((int)(f->time * 10) << 4)
+							| XMP_ECHO_TIME);
 	}
 
-	/* For each frame */
-
-	if (f->ord != p->pos) {			/* changed pattern */
-		if (p->pos == -1)
-			p->pos++;		/* restart module */
-
-		if (p->pos == -2) {		/* set by xmp_module_stop */
-			xmp_drv_bufwipe(ctx);
-			return -1;		/* that's all folks */
-		}
-
-		if (p->pos == 0)
-			f->end_point = p->xmp_scan_num;
-
-		p->tempo = m->xxo_info[f->ord = p->pos].tempo;
-		p->tick_time = m->rrate / (p->xmp_bpm = m->xxo_info[f->ord].bpm);
-		m->volume = m->xxo_info[f->ord].gvl;
-		f->jump = f->ord;
-		f->jumpline = m->xxo_fstrow[f->ord--];
-		f->row = -1;
-		xmp_drv_bufwipe(ctx);
-		xmp_drv_sync(ctx, 0);
-		xmp_drv_reset(ctx);
-		reset_channel(ctx);
-		goto next_row;
-	}
 
 
 	xmp_drv_echoback(ctx, (f->frame << 4) | XMP_ECHO_FRM);
@@ -1155,31 +961,32 @@ int xmp_player_loop(struct xmp_context *ctx)
 	if (o->time && (o->time < f->playing_time))	/* expired time */
 		return -1;
 
-	if (m->fetch & XMP_CTL_MEDBPM) {
+	if (HAS_QUIRK(XMP_QRK_MEDBPM)) {
 		xmp_drv_sync(ctx, p->tick_time * 33 / 125);
 		f->playing_time += m->rrate * 33 / (100 * p->xmp_bpm * 125);
 	} else {
 		xmp_drv_sync(ctx, p->tick_time);
 		f->playing_time += m->rrate / (100 * p->xmp_bpm);
+		f->time += m->rrate / (100 * p->xmp_bpm);
 	}
 
-	xmp_drv_bufdump(ctx);
+	f->frame++;
 
-
-
-	/* check new row */
-	if (f->frame >= (p->tempo * (1 + f->delay)) - 1) {
+	if (f->frame >= (p->tempo * (1 + f->delay))) {
 next_row:
-//printf("** new row\n");
 		f->frame = 0;
 		f->delay = 0;
 
-		if (f->row == -1)
-			goto next_row;
-
 		if (f->pbreak) {
 			f->pbreak = 0;
-			goto next_row;
+
+			if (f->jump != -1) {
+				f->ord = f->jump - 1;
+				f->jump = -1;
+				goto next_order;
+			}
+
+			goto next_order;
 		}
 
 		if (f->loop_chn) {
@@ -1189,28 +996,9 @@ next_row:
 
 		f->row++;
 
-		if (f->jump != -1) {
-			f->ord = f->jump;
-			f->jump = -1;
-			goto next_order;
-		}
-
-#if 0
-		if (p->pause) {
-			xmp_drv_stoptimer(ctx);
-			while (p->pause) {
-				usleep(125000);
-				xmp_event_callback(0);
-			}
-			xmp_drv_starttimer(ctx);
-	    	}
-#endif
-
-
 		/* check end of pattern */
 		if (f->row >= f->num_rows) {
 next_order:
-//printf("*** new order\n");
     			f->ord++;
 
 			/* Restart module */
@@ -1236,23 +1024,22 @@ next_order:
 			p->pos = f->ord;
 
 			/* Reset persistent effects at new pattern */
-			if (m->fetch & XMP_CTL_PERPAT) {
+			if (HAS_QUIRK(XMP_QRK_PERPAT)) {
 				int chn;
 				for (chn = 0; chn < p->m.xxh->chn; chn++)
 					p->xc_data[chn].per_flags = 0;
 			}
 		}
-	} else {
-		f->frame++;
 	}
 
 	return 0;
 }
     
 
-void xmp_player_end(struct xmp_context *ctx)
+void _xmp_player_end(struct xmp_context *ctx)
 {
 	struct xmp_player_context *p = &ctx->p;
+	struct flow_control *f = &p->flow;
 
 	xmp_drv_echoback(ctx, XMP_ECHO_END);
 	while (xmp_drv_getmsg(ctx) != XMP_ECHO_END)
@@ -1266,4 +1053,3 @@ void xmp_player_end(struct xmp_context *ctx)
 
 	xmp_drv_off(ctx);
 }
-#endif
