@@ -9,7 +9,10 @@
 #ifndef __XMP_PLAYER_H
 #define __XMP_PLAYER_H
 
-/* Flag control */
+/* Quirk control */
+#define HAS_QUIRK(x)	(m->quirk & (x))
+
+/* Channel flag control */
 #define SET(f)		SET_FLAG(xc->flags,f)
 #define RESET(f) 	RESET_FLAG(xc->flags,f)
 #define TEST(f)		TEST_FLAG(xc->flags,f)
@@ -88,7 +91,7 @@ struct xmp_channel {
     int per_flags;		/* Persistent effect channel flags */
     int note;			/* Note number */
     int key;			/* Key number */
-    int period;			/* Amiga or linear period */
+    double period;		/* Amiga or linear period */
     int pitchbend;		/* Pitch bender value */
     int finetune;		/* Guess what */
     int ins;			/* Instrument number */
@@ -180,7 +183,7 @@ struct xmp_channel {
 };
 
 
-void process_fx(struct xmp_context *, int, uint8, uint8, uint8, struct xmp_channel *);
+void process_fx(struct xmp_context *, int, uint8, uint8, uint8, struct xmp_channel *, int);
 void xmp_med_synth(struct xmp_context *, int, struct xmp_channel *, int);
 int get_med_arp(struct xmp_player_context *, struct xmp_channel *);
 int get_med_vibrato(struct xmp_channel *);
