@@ -94,16 +94,12 @@ int skip = 0;
 static short audio_open = FALSE;
 
 static gchar *fmts[] = {
-	"xm", "mod", "flt", "st", "m15", "it", "s2m", "s3m", "stm", "stx",
+	"xm", "mod", "m15", "it", "s2m", "s3m", "stm", "stx", "med", "dmf",
 	"mtm", "ice", "imf", "ptm", "mdl", "ult", "liq", "psm", "amf",
-	"mmd0", "mmd1", "mmd2", "mmd3", "med", "med3", "med4", "dmf",
-        "rtm", "pt3", "tcb", "dt", "gtk", "dtt", "mgt", "arch", "dsym",
-	"digi", "dbm", "emod", "okt", "sfx", "far", "umx", "stim",
-	"mtp", "ims", "669", "fnk", "funk", "amd", "rad", "hsc", "alm",
-        "ac1d", "fchs", "fcm", "fuzz", "kris", "ksm", "mp", "p18a",
-	"p10c", "pp21" "pru1", "pru2", "pha", "wn", "unic", "tp3", "xann",
-	"di", "eu", "p4x", "p40", "p41", "p50a" "p60a", "np1", "np2", "np3",
-	"zen", "crb", "tdd", "gmc", "gdm", NULL
+        "rtm", "pt3", "tcb", "dt", "gtk", "dtt", "mgt", "digi", "dbm",
+	"emod", "okt", "sfx", "far", "umx", "stim", "mtp", "ims", "669",
+	"fnk", "funk", "amd", "rad", "hsc", "alm", "kris", "ksm", "unic",
+	"zen", "crb", "tdd", "gmc", "gdm", "mdz", "xmz", "s3z", "j2b", NULL
 };
 
 
@@ -136,6 +132,9 @@ DECLARE_PLUGIN(xmp, NULL, NULL, xmp_iplist, NULL, NULL, NULL, NULL, NULL);
 
 #endif
  
+extern struct xmp_drv_info drv_smix;
+
+
 static void strip_vfs(char *s)
 {
 	int len;
@@ -201,7 +200,7 @@ static void aboutbox()
 		"Portions Copyright (C) 1998,2000 Olivier Lapicque,\n"
 		"(C) 1998 Tammo Hinrichs, (C) 1998 Sylvain Chipaux,\n"
 		"(C) 1997 Bert Jahn, (C) 1999 Tatsuyuki Satoh, (C)\n"
-		"2001-2006 Russell Marks\n"
+		"2001-2006 Russell Marks, (C) 2005-2006 Michael Kohn\n"
 		"\n"
 		"Supported module formats:"
 	);
@@ -318,31 +317,12 @@ InputPlugin *get_iplugin_info()
 #endif
 
 
-static void driver_callback(void *b, int i)
-{
-#if __AUDACIOUS_PLUGIN_API__ >= 2
-	play_data.ipb->pass_audio(play_data.ipb, play_data.fmt, play_data.nch,
-					i, b, &play_data.ipb->playing);
-
-#else
-	xmp_ip.add_vis_pcm(xmp_ip.output->written_time(),
-			xmp_cfg.force8bit ? FMT_U8 : FMT_S16_NE,
-			xmp_cfg.force_mono ? 1 : 2, i, b);
-	
-	while (xmp_ip.output->buffer_free() < i && play_data.ipb->playing)
-		usleep(10000);
-
-	if (play_data.ipb->playing)
-		xmp_ip.output->write_audio(b, i);
-#endif
-}
-
-
 static void init(void)
 {
 	ConfigDb *cfg;
 
 	_D("Plugin init");
+	xmp_drv_register(&drv_smix);
 	ctx = xmp_create_context();
 
 	xmp_cfg.mixing_freq = 0;
@@ -371,7 +351,7 @@ static void init(void)
 		aud_cfg_db_close(cfg);
 	}
 
-	xmp_init_callback(ctx, driver_callback);
+	xmp_init(ctx, 0, NULL);
 }
 
 
@@ -501,7 +481,7 @@ static void play_file(InputPlayback *ipb)
 
 	opt->resol = 8;
 	opt->verbosity = 0;
-	opt->drv_id = "callback";
+	opt->drv_id = "smix";
 
 	switch (xmp_cfg.mixing_freq) {
 	case 1:
@@ -589,8 +569,33 @@ static void play_file(InputPlayback *ipb)
 static gpointer play_loop(gpointer arg)
 {
 	InputPlayback *ipb = arg;
+	void *data;
+	int size;
+
+	xmp_player_start(ctx);
+	while (xmp_player_frame(ctx) == 0) {
+		xmp_get_buffer(ctx, &data, &size);
+
+#if __AUDACIOUS_PLUGIN_API__ >= 2
+		play_data.ipb->pass_audio(play_data.ipb, play_data.fmt,
+			play_data.nch, size, data, &play_data.ipb->playing);
+
+#else
+		xmp_ip.add_vis_pcm(xmp_ip.output->written_time(),
+			xmp_cfg.force8bit ? FMT_U8 : FMT_S16_NE,
+			xmp_cfg.force_mono ? 1 : 2, size, data);
+	
+		while (xmp_ip.output->buffer_free() < size && play_data.ipb->playing)
+			usleep(10000);
+
+		if (play_data.ipb->playing)
+			xmp_ip.output->write_audio(data, size);
+#endif
+	}
+
+
+	xmp_player_end(ctx);
 
-	xmp_play_module(ctx);
 	xmp_release_module(ctx);
 	xmp_close_audio(ctx);
 
