@@ -101,12 +101,10 @@ InputPlugin xmp_ip = {
 	.seek		= seek,
 	.get_time	= get_time,
 	.get_song_info	= get_song_info,
-	//.file_info_box	= file_info_box,
+	/* .file_info_box	= file_info_box, */
 };
 
-
-static void file_info_box_build (void);
-static void init_visual (GdkVisual *);
+extern struct xmp_drv_info drv_smix;
 
 static void aboutbox()
 {
@@ -147,7 +145,7 @@ static void aboutbox()
 		"Portions Copyright (C) 1998,2000 Olivier Lapicque,\n"
 		"(C) 1998 Tammo Hinrichs, (C) 1998 Sylvain Chipaux,\n"
 		"(C) 1997 Bert Jahn, (C) 1999 Tatsuyuki Satoh, (C)\n"
-		"2001-2006 Russell Marks\n"
+		"2001-2006 Russell Marks, (C) 2005-2006 Michael Kohn\n"
 		"\n"
 		"Supported module formats:"
 	);
@@ -258,25 +256,12 @@ InputPlugin *get_iplugin_info()
 }
 
 
-static void driver_callback(void *b, int i)
-{
-	xmp_ip.add_vis_pcm(xmp_ip.output->written_time(),
-			xmp_cfg.force8bit ? FMT_U8 : FMT_S16_NE,
-			xmp_cfg.force_mono ? 1 : 2, i, b);
-	
-	while (xmp_ip.output->buffer_free() < i && playing)
-		usleep(10000);
-
-	if (playing)
-		xmp_ip.output->write_audio(b, i);
-}
-
-
 static void init(void)
 {
 	ConfigFile *cfg;
 	gchar *filename;
 
+	xmp_drv_register(&drv_smix);
 	ctx = xmp_create_context();
 
 	xmp_cfg.mixing_freq = 0;
@@ -307,8 +292,7 @@ static void init(void)
 		xmms_cfg_free(cfg);
 	}
 
-	xmp_init_callback(ctx, driver_callback);
-	xmp_register_event_callback(x11_event_callback);
+	xmp_init(ctx, 0, NULL);
 }
 
 
@@ -358,7 +342,6 @@ static void play_file(char *filename)
 	FILE *f;
 	struct xmp_options *opt;
 	int lret;
-	GtkTextIter start, end;
 	AFormat fmt;
 	int nch;
 	
@@ -374,16 +357,12 @@ static void play_file(char *filename)
 	}
 	fclose(f);
 
-	gtk_text_buffer_get_start_iter(text1b, &start);
-	gtk_text_buffer_get_end_iter(text1b, &end);
-	gtk_text_buffer_delete(text1b, &start, &end);
-	
 	xmp_plugin_audio_error = FALSE;
 	playing = 1;
 
 	opt->resol = 8;
 	opt->verbosity = 0;
-	opt->drv_id = "callback";
+	opt->drv_id = "smix";
 
 	switch (xmp_cfg.mixing_freq) {
 	case 1:
@@ -434,12 +413,6 @@ static void play_file(char *filename)
 
 	xmp_open_audio(ctx);
 
-	pipe(fd_info);
-	fd_old2 = dup (fileno (stderr));
-	dup2(fd_info[1], fileno (stderr));
-	fflush(stderr);
-	pthread_create(&catch_thread, NULL, catch_info, NULL);
-
 	_D("*** loading: %s", filename);
 	pthread_mutex_lock(&load_mutex);
 	lret =  xmp_load_module(ctx, filename);
@@ -451,8 +424,6 @@ static void play_file(char *filename)
 		return;
 	}
 
-	gtk_adjustment_set_value(GTK_TEXT_VIEW(text1)->vadjustment, 0.0);
-
 	xmp_cfg.time = lret;
 	xmp_get_module_info(ctx, &xmp_cfg.mod_info);
 
@@ -463,7 +434,25 @@ static void play_file(char *filename)
 
 static void *play_loop(void *arg)
 {
-	xmp_play_module(ctx);
+	void *data;
+	int size;
+
+	xmp_player_start(ctx);
+	while (xmp_player_frame(ctx) == 0) {
+		xmp_get_buffer(ctx, &data, &size);
+
+		xmp_ip.add_vis_pcm(xmp_ip.output->written_time(),
+			xmp_cfg.force8bit ? FMT_U8 : FMT_S16_NE,
+			xmp_cfg.force_mono ? 1 : 2, size, data);
+	
+		while (xmp_ip.output->buffer_free() < size && playing)
+			usleep(10000);
+
+		if (playing)
+			xmp_ip.output->write_audio(data, size);
+	}
+	xmp_player_end(ctx);
+
 	xmp_release_module(ctx);
 	xmp_close_audio(ctx);
 	playing = 0;
