@@ -64,6 +64,8 @@ static struct xmp_options *opt = NULL;
 
 char mix_buffer[MIX_BUFSIZE * 4];
 
+extern struct xmp_drv_info drv_file;
+
 static void	config		(HWND);
 static void	about		(HWND);
 static void	init		(void);
@@ -90,15 +92,6 @@ In_Module mod = {
 	0,			/* hDllInstance */
 	// Leave out the following more obscure extensions, 
 	// cause overloading GetOpenFileName causes missing extensions
-/*
-	"flt;st;stx;m15;ice;imf;mdl;"
-	"liq;psm;mmd0;mmd1;mmd2;mmd3;med3;med4;dmf;"
-	"rtm;pt3;tcb;dt;gtk;dtt;mgt;arch;dsym;dbm;emod;"
-	"sfx;umx;stim;mtp;ims;fnk;funk;"
-	"ac1d;fchs;fcm;fuzz;kris;ksm;mp;p18a;p10c;pru1;pru2;pha;"
-	"wn;unic;tp3;xann;di;eu;p4x;p40;p41;p60a;np1;np2;np3;zen;"
-	"crb;tdd;gmc;gdm;"
-*/
 	/* file extensions */
 	"xm;it;s3m;s2m;stm;mdl;imf;mtm;ptm;ult;far;669\0"
 		"XMP PC Modules (xm;it;s3m;s2m;stm;mdl;imf;mtm;ptm;ult;far;669)\0"
@@ -106,8 +99,8 @@ In_Module mod = {
 		"XMP Amiga Modules (mod;med;digi;dbm;okt;sfx)\0"
 	"xmz;mdz;itz;s3z;amf\0"
 		"XMP Compressed Modules (xmz;mdz;itz;s3z;amf)\0"
-	"psm;umx;gmc;di;stim\0"
-		"XMP Game Modules (psm;umx;gmc;di;stim)\0"
+	"psm;j2b;umx;gmc;di;stim\0"
+		"XMP Game Modules (psm;j2b;umx;gmc;di;stim)\0"
 	"amd;rad;hsc\0"
 		"XMP Adlib Modules (amd;rad;hsc)\0"
 		,	
@@ -184,44 +177,6 @@ static void stop()
 	mod.SAVSADeInit();
 }
 
-static void driver_callback(void *b, int i)
-{
-	int dsp = mod.dsp_isactive();
-	int n = i * (dsp ? 2 : 1);
-	int numch = opt->outfmt & XMP_FMT_MONO ? 1 : 2;
-	int ssize = opt->resol / 8;
-	int t, todo;
-
-	if (dsp) {
-		char * bb = b;
-		/* Winamp dsp support fixed by Mirko Buffoni */
-		while (i > 0) {
-			todo = (i > MIX_BUFSIZE * 2) ? MIX_BUFSIZE : i;
-			memcpy(mix_buffer, bb, todo);
-
-			while (mod.outMod->CanWrite() < todo * 2)
-				Sleep(20);
-
-			t = mod.outMod->GetWrittenTime();
-			mod.SAAddPCMData(mix_buffer, numch, opt->resol, t);
-			mod.VSAAddPCMData(mix_buffer, numch, opt->resol, t);
-			n = mod.dsp_dosamples((short *)mix_buffer,
-				todo / numch / ssize, opt->resol, numch,
-				opt->freq) * numch * ssize;
-			mod.outMod->Write(mix_buffer, n);
-			i -= todo;
-			bb += todo;
-		}
-	} else {
-		while (mod.outMod->CanWrite() < i)
-			Sleep(50);
-		t = mod.outMod->GetWrittenTime();
-		mod.SAAddPCMData(b, numch, opt->resol, t);
-		mod.VSAAddPCMData(b, numch, opt->resol, t);
-		mod.outMod->Write(b, i);
-	}
-}
-
 static BOOL CALLBACK config_dialog(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
 	char inifile[MAX_PATH];
@@ -345,7 +300,7 @@ static void about(HWND hwndParent)
 		"Portions Copyright (C) 1998,2000 Olivier Lapicque,\n"
 		"(C) 1998 Tammo Hinrichs, (C) 1998 Sylvain Chipaux,\n"
 		"(C) 1997 Bert Jahn, (C) 1999 Tatsuyuki Satoh, (C)\n"
-		"2001-2006 Russell Marks\n"
+		"2001-2006 Russell Marks, (C) 2005-2006 Michael Kohn\n"
 		, "About XMP", MB_OK);
 }
 
@@ -354,6 +309,7 @@ static void init()
 {
 	static char inifile[MAX_PATH];
 
+	xmp_drv_register(&drv_smix);
 	ctx = xmp_create_context();
 
 #define CFGREADINT(x,y) do { \
@@ -371,8 +327,7 @@ static void init()
 	CFGREADINT(filter, 1);
 	CFGREADINT(pan_amplitude, 80);
 
-	xmp_init_callback(ctx, driver_callback);
-	//xmp_register_event_callback(x11_event_callback);
+	xmp_init(ctx, 0, NULL);
 }
 
 static void quit()
@@ -412,7 +367,7 @@ static int play_file(char *fn)
 
 	opt->resol = 16;
 	opt->verbosity = 0;
-	opt->drv_id = "callback";
+	opt->drv_id = "smix";
 
 	opt->freq = xmp_cfg.mixing_freq;
 
@@ -483,11 +438,54 @@ static int play_file(char *fn)
 	return 0;
 }
 
-static DWORD WINAPI __stdcall play_loop(void *b)
+static DWORD WINAPI __stdcall play_loop(void *x)
 {
-	_D(_D_CRIT "play");
-	xmp_play_module(ctx);
-	_D(_D_CRIT "end play");
+	int n, dsp;
+	int numch = opt->outfmt & XMP_FMT_MONO ? 1 : 2;
+	int ssize = opt->resol / 8;
+	int t, todo;
+	void *data;
+	int size;
+
+	xmp_player_start(ctx);
+	while (xmp_player_frame(ctx) == 0) {
+		xmp_get_buffer(ctx, &data, &size);
+
+		dsp = mod.dsp_isactive();
+		n = size * (dsp ? 2 : 1);
+
+		if (dsp) {
+			/* Winamp dsp support fixed by Mirko Buffoni */
+			while (size > 0) {
+				todo = (size > MIX_BUFSIZE * 2) ?
+					MIX_BUFSIZE : size;
+				memcpy(mix_buffer, data, todo);
+	
+				while (mod.outMod->CanWrite() < todo * 2)
+					Sleep(20);
+	
+				t = mod.outMod->GetWrittenTime();
+				mod.SAAddPCMData(mix_buffer, numch,
+							opt->resol, t);
+				mod.VSAAddPCMData(mix_buffer, numch,
+							opt->resol, t);
+				n = mod.dsp_dosamples((short *)mix_buffer,
+					todo / numch / ssize, opt->resol, numch,
+					opt->freq) * numch * ssize;
+				mod.outMod->Write(mix_buffer, n);
+				size -= todo;
+				data += todo;
+			}
+		} else {
+			while (mod.outMod->CanWrite() < size)
+				Sleep(50);
+			t = mod.outMod->GetWrittenTime();
+			mod.SAAddPCMData(data, numch, opt->resol, t);
+			mod.VSAAddPCMData(data, numch, opt->resol, t);
+			mod.outMod->Write(data, size);
+		}
+	}
+
 	xmp_release_module(ctx);
 	xmp_close_audio(ctx);
 	playing = 0;
