@@ -103,6 +103,8 @@ InputPlugin xmp_ip = {
 	.get_song_info	= get_song_info,
 };
 
+extern struct xmp_drv_info drv_smix;
+
 
 static void aboutbox()
 {
@@ -143,7 +145,7 @@ static void aboutbox()
 		"Portions Copyright (C) 1998,2000 Olivier Lapicque,\n"
 		"(C) 1998 Tammo Hinrichs, (C) 1998 Sylvain Chipaux,\n"
 		"(C) 1997 Bert Jahn, (C) 1999 Tatsuyuki Satoh, (C)\n"
-		"2001-2006 Russell Marks\n"
+		"2001-2006 Russell Marks, (C) 2005-2006 Michael Kohn\n"
 		"\n"
 		"Supported module formats:"
 	);
@@ -252,25 +254,12 @@ InputPlugin *get_iplugin_info()
 }
 
 
-static void driver_callback(void *b, int i)
-{
-	xmp_ip.add_vis_pcm(xmp_ip.output->written_time(),
-			xmp_cfg.force8bit ? FMT_U8 : FMT_S16_NE,
-			xmp_cfg.force_mono ? 1 : 2, i, b);
-	
-	while (xmp_ip.output->buffer_free() < i && playing)
-		usleep(10000);
-
-	if (playing)
-		xmp_ip.output->write_audio(b, i);
-}
-
-
 static void init(void)
 {
 	ConfigFile *cfg;
 	gchar *filename;
 
+	xmp_drv_register(&drv_smix);
 	ctx = xmp_create_context();
 
 	xmp_cfg.mixing_freq = 0;
@@ -301,7 +290,7 @@ static void init(void)
 		xmms_cfg_free(cfg);
 	}
 
-	xmp_init_callback(ctx, driver_callback);
+	xmp_init(ctx, 0, NULL);
 }
 
 
@@ -371,7 +360,7 @@ static void play_file(char *filename)
 
 	opt->resol = 8;
 	opt->verbosity = 0;
-	opt->drv_id = "callback";
+	opt->drv_id = "smix";
 
 	switch (xmp_cfg.mixing_freq) {
 	case 1:
@@ -447,7 +436,25 @@ static void play_file(char *filename)
 
 static void *play_loop(void *arg)
 {
-	xmp_play_module(ctx);
+	void *data;
+	int size;
+
+	xmp_player_start(ctx);
+	while (xmp_player_frame(ctx) == 0) {
+                xmp_get_buffer(ctx, &data, &size);
+
+		xmp_ip.add_vis_pcm(xmp_ip.output->written_time(),
+			xmp_cfg.force8bit ? FMT_U8 : FMT_S16_NE,
+			xmp_cfg.force_mono ? 1 : 2, i, b);
+	
+		while (xmp_ip.output->buffer_free() < i && playing)
+			usleep(10000);
+
+		if (playing)
+			xmp_ip.output->write_audio(b, i);
+	}
+        xmp_player_end(ctx);
+
 	xmp_release_module(ctx);
 	xmp_close_audio(ctx);
 	playing = 0;
